# prep MODIS MOD data for gapfill
# mosaic


library(raster)
library(fs)
library(magrittr)
library(foreach)
library(data.table)
library(lubridate)



# get files ---------------------------------------------------------------

path_in <- "/mnt/CEPH_PROJECTS/SNOW_3rd_gen/MOD/"

files_all <- dir_ls(path_in, recurse = T, glob = "*EURAC*.tif")

dat <- data.table(file_full = files_all,
                  file_base = path_file(files_all))
dat[, paste0("col", 1:10) := tstrsplit(file_base, "[.]")]
setnames(dat, c("col3", "col4", "col6"), c("tile", "year", "doy"))

dat[, date := make_date(year)]
yday(dat$date) <- as.numeric(dat$doy)

dat



# aux only ----------------------------------------------------------------

rr_aux_18 <- stack("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/MOD/aux_snow_all_alps_sinosoidal_dem_h18_006.tif")
rr_aux_19 <- stack("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/MOD/aux_snow_all_alps_sinosoidal_dem_h19.tif")

rr_aux_out <- mosaic(rr_aux_18, rr_aux_19, fun = min)

writeRaster(rr_aux_out, 
            "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/00_aux/eurac_modis_aux_sinu.tif",
            overwrite = T)


# check input -------------------------------------------------------------

dat[, .N, tile]
with(dat, table(tile, year))
dat[year >= 2019 & tile == "h18v04"]


# loop - mosaic -----------------------------------------------------------


dates_all <- sort(unique(dat$date))[1:7091]
path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/01_original/"


mitmatmisc::init_parallel_ubuntu()

foreach(
  i_date = dates_all,
  .inorder = F
) %dopar% {
  
  fn_out <- path(path_out,
                 paste0(format(i_date, "%Y%m%d"), "_120000.tif"))
  if(file_exists(fn_out)) {
    cat("Skipped (file exists):", fn_out, "\n")
    return(NULL)
  }
  
  fn_18 <- dat[tile == "h18v04" & date == i_date, file_full]
  fn_19 <- dat[tile == "h19v04" & date == i_date, file_full]

  rr_18 <- raster(fn_18)
  rr_19 <- raster(fn_19)
  
  rr_out <- mosaic(rr_18, rr_19, fun = min)
  
  writeRaster(rr_out, fn_out)
  cat("Done:", fn_out, "\n")
  return(NULL)
}




