# extract data.table of SNC from upscaled MODIS
# ----------------------------- #

## prereq ####

source("R/functions/raster2datatable.R")

library(raster)
library(data.table)
library(magrittr)
library(lubridate)
library(stringr)


path_in <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/daily/modis/"
path_out <- ""


# get path and filenames
all_files <- list.files(path_in)

# get parallel
library(doParallel)
library(foreach)
registerDoParallel(parallel::detectCores() - 1)

## loop over all dates (in parallel) ####
l <- foreach(fn = all_files, 
             .inorder = F,
             .packages = c("raster", "data.table", "lubridate",
                           "magrittr", "stringr")
) %dopar% {
  
  # extract date from filename
  fn_date <- str_extract(fn, "\\d{4}-\\d{2}-\\d{2}")
  
  # open brick
  rr_modis <- brick(file.path(path_in, fn))
  
  # transform into data.table
  rr_modis %>% 
    raster2datatable(xy = F, icell = T, remove.na = T) -> dt_res
  
  # setnames
  setnames(dt_res, c("snow", "no.snow", "clouds", "icell"))
  
  # add IDate
  dt_res[, date := as.IDate(ymd(fn_date))]
  
  # return
  dt_res
}

# rbind
dt_modis <- rbindlist(l)

# save
saveRDS(dt_modis,
        file = "data/modis/01-modis-snow-nosnow-clouds.rds")
