# use CDO to:
#   remap selected grid inventory
#   MODIS time frame: 2000-2019
#                     and also future?
#   variables: snc, orog
#
# ------------------------- #

library(data.table)
library(magrittr)
library(fs)
# get parallel
library(doParallel)
library(foreach)
registerDoParallel(parallel::detectCores() - 1)

# set remapping algorithm:
# NEEDS TO BE CONSTANT FOR ALL VARIABLES!!
# otherwise if bil for temp and nn for snc, then interpolation artifacts?!
# hard coded to nn, because bilinear introduces inplausibilities with constant 0 and 100 snc
# remap_type <- "remapnn" # nearest-neighbour
# remap_type <- "remapbil" # bilinear


## SNC and TAS; 2000-2019; daily and monthly ####

# remap function with date selection (for SNC, TAS, ...)
f_remap <- function(dat_i, path_out_daily, path_out_monthly){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    # get and set filename
    files_original <- dat_i$list_files[i][[1]]
    
    # remap & monmean proc
    foreach(j = seq_along(files_original)) %do% {
      
      # daily
      in_file <- files_original[j]
      out_file <- path(path_out_daily, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
    
      cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                         " ", "-seldate,20000101,20191231",
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      add_return <- paste0("Done: ", out_file, "\n")
      
      # make monthly
      in_file <- out_file
      if(!file_exists(in_file)) return(paste0(add_return, "Skipped monthly: ", in_file)) # no input
      out_file <- path(path_out_monthly, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0(add_return, "Skipped: ", out_file))
      
      cdo_call <- paste0("cdo monmean", 
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      return(paste0(add_return, "Done: ", out_file))
    }
  }
}



# # # snc
# f_remap(dat_i = readRDS("data/eurocordex-inventory/grid/snc.rds"),
#         path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/",
#         path_out_monthly = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/monthly/rcm/")
# 
# # # tas
# f_remap(dat_i = readRDS("data/eurocordex-inventory/grid/tas.rds"),
#         path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/",
#         path_out_monthly = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/monthly/rcm/")
# 


## OROG ####


# remap function for fx variables (OROG, ...)
f_remap_fx <- function(dat_i, path_out){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    in_file <- dat_i$list_files[i][[1]]
    out_file <- path(path_out, 
                     dat_i[i, variable], 
                     path_file(in_file))
    
    if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
    if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
    
    cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                       " ", in_file, 
                       " ", out_file)
    system(cdo_call)
    return(paste0("Done: ", out_file))
  }
}




# orog
# f_remap_fx(dat_i = readRDS("data/eurocordex-inventory/orog.rds"),
#            path_out = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/fixed/rcm/")
# 




## SNC, SND, SNW; only 2000 and only daily ####

# remap function with date selection (for SNC, TAS, ...)
f_remap_intercomp <- function(dat_i, path_out_daily){

  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    # get and set filename
    files_original <- dat_i$list_files[i][[1]]
    
    # remap & monmean proc
    foreach(j = seq_along(files_original)) %do% {
      
      # daily
      in_file <- files_original[j]
      out_file <- path(path_out_daily, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
      
      cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                         " ", "-seldate,20000101,20001231",
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      return(paste0("Done: ", out_file))
      
    }
  }
}



# snc, snd, snw
# f_remap_intercomp(dat_i = readRDS("data/eurocordex-inventory/grid/intercomp-snc-snd-snw.rds"),
#                   path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/intercomp-2000/")



## maximum SND and SNW ####


# remap function with tim* aggregation
f_remap_tim <- function(dat_i, path_out_daily, agg){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    # get and set filename
    files_original <- dat_i$list_files[i][[1]]
    
    # remap & monmean proc
    foreach(j = seq_along(files_original)) %do% {
      
      # daily
      in_file <- files_original[j]
      out_file <- path(path_out_daily, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
      
      cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                         " ", "-tim", agg,
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      return(paste0("Done: ", out_file))
      
    }
  }
}

# snc, snd, snw
# f_remap_tim(dat_i = readRDS("data/eurocordex-inventory/grid/intercomp-snc-snd-snw.rds"),
#             path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/max-snow/",
#             agg = "max")
# f_remap_tim(dat_i = readRDS("data/eurocordex-inventory/grid/intercomp-snc-snd-snw.rds"),
#             path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/min-snow/",
#             agg = "min")


## maximum snow (rotpole) ####



# only tim* aggregation
f_sellonlat_tim <- function(dat_i, path_out_daily, agg){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    # get and set filename
    files_original <- dat_i$list_files[i][[1]]
    
    # remap & monmean proc
    foreach(j = seq_along(files_original)) %do% {
      
      # daily
      in_file <- files_original[j]
      out_file <- path(path_out_daily, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
      
      cdo_call <- paste0("cdo ", "sellonlatbox,0,20,40,50", 
                         " ", "-tim", agg,
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      return(paste0("Done: ", out_file))
      
    }
  }
}

# snc, snd, snw
# f_sellonlat_tim(dat_i = readRDS("data/eurocordex-inventory/grid/intercomp-snc-snd-snw.rds"),
#            path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/max-snow-rotpole/",
#            agg = "max")



## SNC, SND, SNW; 1971 - 2000, only daily, only final gcm-rcm ####


f_remap_intercomp2 <- function(dat_i, path_out_daily){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    # get and set filename
    files_original <- dat_i$list_files[i][[1]]
    
    # remap & monmean proc
    foreach(j = seq_along(files_original)) %do% {
      
      # daily
      in_file <- files_original[j]
      out_file <- path(path_out_daily, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
      
      cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                         " ", "-seldate,19710101,20001231",
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      return(paste0("Done: ", out_file))
      
    }
  }
}

# subset to needed gcm-rcm
dat_i <- readRDS("data/eurocordex-inventory/grid/intercomp-snc-snd-snw.rds")
dat_i <- dat_i[(gcm == "CNRM-CERFACS-CNRM-CM5" & institute_rcm %in% c("CLMcom-CCLM4-8-17",
                                                                      "CNRM-ALADIN63",
                                                                      "DMI-HIRHAM5",
                                                                      "KNMI-RACMO22E")) |
                 (gcm == "IPSL-IPSL-CM5A-MR" & institute_rcm %in% c("IPSL-INERIS-WRF331F",
                                                                    "IPSL-WRF381P") ) | 
                 (gcm == "MPI-M-MPI-ESM-LR" & institute_rcm == "SMHI-RCA4")]



# snc, snd, snw
f_remap_intercomp2(dat_i = dat_i,
                  path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/intercomp-1971-2000-sub/")



## PR; 2000-2019; daily and monthly ####

# remap function with date selection (for SNC, TAS, ...)
f_remap <- function(dat_i, path_out_daily, path_out_monthly){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    # get and set filename
    files_original <- dat_i$list_files[i][[1]]
    
    # remap & monmean proc
    foreach(j = seq_along(files_original)) %do% {
      
      # daily
      in_file <- files_original[j]
      out_file <- path(path_out_daily, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
      
      cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                         " ", "-seldate,20000101,20191231",
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      add_return <- paste0("Done: ", out_file, "\n")
      
      # make monthly
      in_file <- out_file
      if(!file_exists(in_file)) return(paste0(add_return, "Skipped monthly: ", in_file)) # no input
      out_file <- path(path_out_monthly, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0(add_return, "Skipped: ", out_file))
      
      cdo_call <- paste0("cdo monsum", 
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      return(paste0(add_return, "Done: ", out_file))
    }
  }
}



# pr
f_remap(dat_i = readRDS("data/eurocordex-inventory/grid/pr.rds"),
        path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/",
        path_out_monthly = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/monthly/rcm/")



## EOF ####