# extract the monthly RCM grid to rds ####

library(eurocordexr)
library(foreach)


dat_inv <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/monthly/rcm/", add_files = T)


## eval first ####
dat_inv_eval <- dat_inv[experiment == "evaluation"]
all_vars <- unique(dat_inv_eval$variable)

dat_eval <- foreach(
  i_var = all_vars,
  .final = function(x) Reduce(merge, x)
) %do% {
  
  dat_i <- dat_inv_eval[variable == i_var]
  
  foreach(
    i_row = 1:nrow(dat_i),
    .final = rbindlist
  ) %do% {
    
    foreach(
      i_file = dat_i[i_row, list_files][[1]],
      .final = rbindlist
    ) %do% {
      
      dat_nc <- nc_grid_to_dt(i_file, i_var)
      dat_nc[, gcm := dat_i[i_row, gcm]]
      dat_nc[, rcm := dat_i[i_row, institute_rcm]]
      dat_nc[, experiment := dat_i[i_row, experiment]]

      dat_nc
      
    }
    
  }
  
}

# adjust units
dat_eval[, tas := tas - 273.15]
dat_eval[, snc := snc/100]
dat_eval[, pr := pr * 24*60*60]

saveRDS(dat_eval, file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/grid/01-evaluation-monthly.rds")

## all other ####

dat_inv_other <- dat_inv[experiment != "evaluation"]
all_vars <- unique(dat_inv_other$variable)

dat_other <- foreach(
  i_var = all_vars,
  .final = function(x) Reduce(merge, x)
) %do% {

  dat_i <- dat_inv_other[variable == i_var]

  dat_i_full <- foreach(
    i_row = 1:nrow(dat_i),
    .final = rbindlist
  ) %do% {

    foreach(
      i_file = dat_i[i_row, list_files][[1]],
      .final = rbindlist
    ) %do% {

      dat_nc <- nc_grid_to_dt(i_file, i_var)
      dat_nc[, gcm := dat_i[i_row, gcm]]
      dat_nc[, rcm := dat_i[i_row, institute_rcm]]
      dat_nc[, experiment := dat_i[i_row, experiment]]
      
      # some have non-standard calendars, but for monthly data does not matter
      dat_nc[, date := ymd(date)]
      
      dat_nc

    }

  }
  
  # copy historical to each rcp 
  l_gcmrcm <- split(dat_i_full, by = c("gcm", "rcm"))  
  dat_plus <- rbindlist(lapply(l_gcmrcm, function(dat_i_gcmrcm){
    dat_i_hist <- dat_i_gcmrcm[experiment == "historical"]
    dat_i_plus <- foreach(
      i_exp = dat_i_gcmrcm[experiment != "historical", unique(experiment)],
      .final = rbindlist
    ) %do% {
      dat_out <- rbind(dat_i_hist, dat_i_gcmrcm[experiment == i_exp])
      dat_out[, experiment := paste0("hist+", i_exp)]
      dat_out
    }
    dat_i_plus
  }))
  
  dat_plus

}


# adjust units
dat_other[, tas := tas - 273.15]
dat_other[, snc := snc/100]
dat_other[, pr := pr*24*60*60]


saveRDS(dat_other, file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/grid/02-hist-rcp-monthly.rds")




