# some example results snow cover fraction by region

library(foreach)
library(data.table)
library(raster)
library(lubridate)
library(magrittr)
library(ggplot2)
library(forcats)

ncell <- 14051167

## read/prep data ####

all_files <- list.files("data-orig/freq-v4-region/", full.names = T)
dat_stats <- foreach(
  i_file = all_files,
  .final = rbindlist
) %do% {
  
  dat <- fread(i_file)
  
  dat[, .(ff = basename(i_file) %>% gsub(".csv", "", .),
          region,
          date = as.Date(ymd_hms(file)),
          value, 
          count)]
  
}

# summarize nodata/water into 1 class

dat_stats[, count_frac := count / ncell]
dat_stats[, value_f := fct_collapse(as.character(value),
                                    nodata_water = c("0", "4", "5"),
                                    snow = "1",
                                    land = "2",
                                    clouds = "3")]
dat_stats[, region_f := factor(region)]
dat_stats[, region_f := forcats::fct_recode(region_f, 
                                                   SW = "1", NW = "2", NE = "3", SE = "4")]

dat_stats_class <- dat_stats[, 
                             .(count_frac = sum(count_frac),
                               count_sum = sum(count)),
                             .(ff, region_f, date, value_f)]

# fill in 0's for missing dates 

CJ(ff = unique(dat_stats_class$ff),
   region_f = unique(dat_stats_class$region_f),
   value_f = unique(dat_stats_class$value_f),
   date = seq(min(dat_stats$date), max(dat_stats$date), by = "day")) %>% 
  merge(dat_stats_class, all.x = T) -> dat_stats_class_full
# with(dat_stats_class_full, table(ff, value_f, is.na(count_frac)))

dat_stats_class_full[is.na(count_frac), count_frac := 0]
dat_stats_class_full[is.na(count_sum), count_sum := 0]



# plot --------------------------------------------------------------------


dat_stats_class_full[, month_f := factor(month.abb[month(date)],
                             levels = month.abb[c(10:12, 1:9)])]
dat_stats_class_full[, year := year(date)]
dat_stats_class_full[, hydro_year := year]
dat_stats_class_full[month(date) <= 9, hydro_year := hydro_year - 1L]
dat_stats_class_full[, .N, hydro_year]

# dat_plot_year <- dat_stats_class_full[value_f == "snow" & hydro_year > 2001 & hydro_year < 2018,
#                      .(ncell_snow = mean(count_sum)),
#                      .(region_f, hydro_year)]


dat_plot_year <- dat_stats_class_full[value_f == "snow" & month_f %in% c("Jan"),
                                      .(ncell_snow = mean(count_sum)),
                                      .(region_f, month_f, year)]

dat_plot_year[, region_f2 := fct_relevel(region_f, c("NW", "NE", "SW", "SE"))]

# get real area
rr0 <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/01_download/20020704_120000.tif")
cellsize_km2 <- prod(res(rr0)/1000) # km2
dat_plot_year[, area_snow := ncell_snow * cellsize_km2]

dat_plot_year %>% 
  ggplot(aes(year, area_snow))+
  geom_line()+
  # geom_smooth(method = lm)+
  # facet_grid(month_f~region_f2, scales = "free_y")+
  facet_wrap(~region_f2)+
  theme_bw()+
  xlab(NULL)+
  ylab(expression(paste("Snow covered area [", km^2, "]")))

ggsave(filename = "fig/review-01/Figure 2.png",
       width = 8, height = 5, units = "in")

