

## -------------------------------------------- ##
## INPUT ####
## -------------------------------------------- ##

library(data.table)
library(eurocordexr)
library(doParallel)
library(foreach)


# prep station info (name, lat lon) 
dat_stn_info_all <- readRDS("data-aux/stn/00-stn-info.rds")
load("data-aux/stn/07-subset2-4x-monthly.rda")
all_stns <- sort(unique(c(dat_stn_gcm_full$stn_id,
                          dat_stn_gcm_sub$stn_id,
                          dat_stn_eval_full$stn_id,
                          dat_stn_eval_sub$stn_id)))

dat_info <- dat_stn_info_all[stn_id %in% all_stns, 
                             .(station = stn_name,
                               lon = long,
                               lat = lat)]

dat_info[, station_file := gsub("[^[:alnum:]]+", "_", station)]


# dat_info <- readRDS(file_info)
dat_inv <- readRDS("data/eurocordex-inventory/point/snc-snd-snw-tas.rds")

path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/point/01-raw/"

# subset to missing stns
stns_existing <- list.files(path_out)
dat_info <- dat_info[!station_file %in% stns_existing]

## -------------------------------------------- ##
## OUTPUT ####
## -------------------------------------------- ##

# *.rds files in path_out of the following structure
# : path_out/[station]/[variable]/[gcm]_[institute_rcm]_[experiment].rds

# each file contains a data.table, see
# ?eurocordexr::rotpole_nc_point_to_dt



## -------------------------------------------- ##
## extract all variables in inventory for all stations ####
## -------------------------------------------- ##

# check inventory
# if(ignore_checks){
#   print("Ignoring checks.")
# } else {
#   all_check_passed <- check_inventory(dat_inv)
#   if(!all_check_passed) stop("Some checks failed in eurocordexr::check_inventory()")
# }



# get parallel
registerDoParallel(parallel::detectCores() - 1)


# extract data and save files
foreach(
  i_stn = dat_info$station_file,
  i_lon = dat_info$lon,
  i_lat = dat_info$lat
) %do% {
  
  foreach(i_nc = 1:nrow(dat_inv), .inorder = F) %dopar% {
    
    file_to_save <- file.path(path_out, 
                              i_stn, 
                              dat_inv[i_nc, variable], 
                              dat_inv[i_nc, paste0(gcm, "_",
                                                   institute_rcm, "_",
                                                   experiment, "_",
                                                   ensemble, "_",
                                                   downscale_realisation, "_",
                                                   timefreq, "_",
                                                   domain,
                                                   ".rds")])
    
    if(!dir.exists(dirname(file_to_save))) dir.create(dirname(file_to_save), recursive = T)
    if(file.exists(file_to_save)) return(paste0("Skipping because file already exists: ",
                                                basename(file_to_save)))
    
    dat_nc <- foreach(
      fn = dat_inv[i_nc, unlist(list_files)],
      .final = rbindlist
    ) %do% {
      
      dat <- rotpole_nc_point_to_dt(filename = fn,
                                    variable = dat_inv[i_nc, variable],
                                    point_lon = i_lon,
                                    point_lat = i_lat,
                                    interpolate_to_standard_calendar = T,
                                    verbose = F)
      
      dat
      
      
    }
    
    saveRDS(dat_nc,
            file = file_to_save)
    
    return(paste0("Done: ", basename(file_to_save)))
  }
  
  return(paste0("Done with all variables and models: ", i_stn))
}