# extract model orography for station (and grid cell coord)

library(data.table)
library(eurocordexr)
library(doParallel)
library(foreach)


# prep station info (name, lat lon) 
dat_stn_info_all <- readRDS("data-aux/stn/00-stn-info.rds")
load("data-aux/stn/07-subset2-4x-monthly.rda")
all_stns <- sort(unique(c(dat_stn_gcm_full$stn_id,
                          dat_stn_gcm_sub$stn_id,
                          dat_stn_eval_full$stn_id,
                          dat_stn_eval_sub$stn_id)))

dat_info <- dat_stn_info_all[stn_id %in% all_stns, 
                             .(station = stn_name,
                               lon = long,
                               lat = lat)]

dat_info[, station_file := gsub("[^[:alnum:]]+", "_", station)]


# dat_info <- readRDS(file_info)
dat_inv <- readRDS("data/eurocordex-inventory/orog.rds")


# get parallel
registerDoParallel(parallel::detectCores() - 1)


# extract data and save files
dat_stn_orog <- foreach(
  i_stn = dat_info$station_file,
  i_lon = dat_info$lon,
  i_lat = dat_info$lat,
  .inorder = F,
  .final = rbindlist
) %dopar% {
  
  dat_one_stn <- foreach(
    i_nc = 1:nrow(dat_inv),
    .final = rbindlist
  ) %do% {
    
    fn <- dat_inv[i_nc, unlist(list_files)]
    
    dat <- rotpole_nc_point_to_dt(filename = fn,
                                  variable = dat_inv[i_nc, variable],
                                  point_lon = i_lon,
                                  point_lat = i_lat,
                                  interpolate_to_standard_calendar = T,
                                  verbose = F,
                                  add_grid_coord = T)
    
    dat[, date := NULL]
    
    dat <- cbind(dat, dat_inv[i_nc, .(gcm, rcm = institute_rcm, experiment)])
    
    dat
    
  }
  
  dat_one_stn[, ":="(station_file = i_stn,
                     stn_lon = i_lon,
                     stn_lat = i_lat)]
  
  dat_one_stn
  
}

saveRDS(dat_stn_orog, file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/point/03-orog.rds")
