# compare bc data with amundsen


library(data.table)
library(magrittr)
library(fs)
library(ggplot2)
library(forcats)
library(scico)

dat_aux_modis <- readRDS("data/auxiliary-03-modis.rds")
dat_aux_amundsen <- readRDS("data/auxiliary-04-amundsen.rds")
dat_amundsen <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/amundsen-02-full.rds")
dat_bc <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/amundsen-03-bc-data.rds")

# subset to common periods
dat_bc2 <- dat_bc[date >= min(dat_amundsen$date)]
dat_amundsen2 <- dat_amundsen[date %in% unique(dat_bc2$date)]

# merge -------------------------------------------------------------------

setnames(dat_amundsen2, "icell", "icell_amundsen")
dat_amundsen2 <- merge(dat_amundsen2, dat_aux_amundsen[, .(icell_amundsen, icell_modis)])
setnames(dat_amundsen2, "icell_modis", "icell")
dat_amundsen2[, month := month(date)]
dat_merge <- merge(dat_bc2, dat_amundsen2, all = T,
                   by = c("rcm", "gcm", "experiment","icell", "date", "month"))
dat_merge[, fp := ifelse(date <= make_date(2021, 1, 1), "past", "future")]

# clim maps past future ---------------------------------------------------

dat_clim <- dat_merge[experiment == "rcp85", 
                      lapply(.SD, mean, na.rm = T),
                      .(rcm, gcm, experiment, icell, month, fp),
                      .SDcols = c("snc", "snc_qm", "snc_qdm", "snc_mbc", "swe", "scf")]

dat_clim %>% 
  melt(measure.vars = c("snc", "snc_qm", "snc_qdm", "snc_mbc", "scf"),
       id.vars = c("rcm", "gcm", "experiment", "icell", "month", "fp")) %>% 
  merge(dat_aux_modis, by = "icell") -> dat_plot

dat_plot[month == 12, .N, .(gcm, rcm, experiment)]
dat_plot[month == 12 & rcm == "IPSL-INERIS-WRF331F"] %>% 
  ggplot(aes(x, y, fill = value))+
  geom_raster()+
  scale_fill_viridis_c(limits = c(0,1))+
  facet_grid(fp ~ variable)+
  theme_bw()+
  coord_equal()

dat_plot[month == 4 & fp == "future"] %>% 
  ggplot(aes(x, y, fill = value))+
  geom_raster()+
  scale_fill_viridis_c(limits = c(0,1))+
  facet_grid(variable ~ rcm + gcm)+
  theme_bw()+
  coord_equal()



# compare area averages --------------------------------------------------------


dat_merge[experiment == "rcp85", 
          lapply(.SD, mean, na.rm = T),
          .(rcm, gcm, experiment, icell, month, fp),
          .SDcols = c("snc", "snc_qm", "snc_qdm", "snc_mbc", "swe", "scf")] %>% 
  na.omit %>% 
  .[,
    lapply(.SD, mean),
    .(rcm, gcm, experiment, month, fp),
    .SDcols = c("snc", "snc_qm", "snc_qdm", "snc_mbc", "swe", "scf")] -> dat_plot_avg

dat_plot_avg[, .N, .(rcm, gcm, fp)]

dat_plot_avg %>% 
  melt(measure.vars = c("snc", "snc_qm", "snc_qdm", "snc_mbc", "scf"),
       id.vars = c("rcm", "gcm", "experiment","month", "fp")) -> dat_plot_avg2

dat_plot_avg %>% 
  melt(measure.vars = c("snc", "snc_qm", "scf"),
       id.vars = c("rcm", "gcm", "experiment","month", "fp")) -> dat_plot_avg2

mitmatmisc::add_month_fct(dat_plot_avg2, 10)

dat_plot_avg2 %>% 
  ggplot(aes(month_fct, value, colour = variable))+
  geom_line(aes(group = paste0(variable,fp), linetype = fp))+
  facet_wrap(~rcm + gcm)+
  theme_bw()

dat_plot_avg[rcm == "IPSL-INERIS-WRF331F"] %>% 
  ggplot(aes(month_fct, value, colour = variable))+
  geom_line(aes(group = paste0(variable,fp), linetype = fp))+
  facet_wrap(~variable)+
  theme_bw()


# -> quite some differences, especially in the melting
# -> but already differences in past, when starting the future simulation


# elevation amundsen, modis, rcm --------------------------------------------



# compare past amundsen vs modis ------------------------------------------








