# compare the different BC routines


library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(ggplot2)
library(forcats)






# some example maps -------------------------------------------------------

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/clim-01.rda")

# mean
dat_plot <- rbindlist(use.names = T, fill = T, list(
  clim_modis[, .(fcol = "MODIS", frow = "past",
                 rcm = NA_character_, icell, month, value = snc)],
  clim_qdm[, .(fcol = "raw", frow = fp, 
               rcm, icell, month, value = snc)],
  dat_delta2[, .(fcol = "delta-change", frow = "future",
                 rcm, icell, month, value = snc_future_delta)],
  clim_qm[, .(fcol = "QM", frow = fp, 
              rcm,  icell, month, value = snc_qm)],
  clim_qdm[, .(fcol = "QDM", frow = fp, 
               rcm,  icell, month, value = snc_qdm)],
  clim_mbc[, .(fcol = "MBC-QDM", frow = fp, 
               rcm,  icell, month, value = snc_qdm)]
))

# quantile = 0.1
dat_plot <- rbindlist(use.names = T, fill = T, list(
  clim_modis[, .(fcol = "MODIS", frow = "past",
                 rcm = NA_character_, icell, month, value = snc_q10)],
  clim_qdm[, .(fcol = "raw", frow = fp, 
               rcm, icell, month, value = snc_q10)],
  dat_delta2[, .(fcol = "delta-change", frow = "future",
                 rcm, icell, month, value = snc_q10_future_delta)],
  clim_qm[, .(fcol = "QM", frow = fp, 
              rcm,  icell, month, value = snc_qm_q10)],
  clim_qdm[, .(fcol = "QDM", frow = fp, 
               rcm,  icell, month, value = snc_qdm_q10)],
  clim_mbc[, .(fcol = "MBC-QDM", frow = fp, 
               rcm,  icell, month, value = snc_qdm_q10)]
))


# quantile = 0.9
dat_plot <- rbindlist(use.names = T, fill = T, list(
  clim_modis[, .(fcol = "MODIS", frow = "past",
                 rcm = NA_character_, icell, month, value = snc_q90)],
  clim_qdm[, .(fcol = "raw", frow = fp, 
               rcm, icell, month, value = snc_q90)],
  dat_delta2[, .(fcol = "delta-change", frow = "future",
                 rcm, icell, month, value = snc_q90_future_delta)],
  clim_qm[, .(fcol = "QM", frow = fp, 
              rcm,  icell, month, value = snc_qm_q90)],
  clim_qdm[, .(fcol = "QDM", frow = fp, 
               rcm,  icell, month, value = snc_qdm_q90)],
  clim_mbc[, .(fcol = "MBC-QDM", frow = fp, 
               rcm,  icell, month, value = snc_qdm_q90)]
))


# for all:
rcm_short <- readRDS("data/auxilary-01-rcm-names.rds")
dat_plot[, rcm_short := rcm_short[rcm]]

dat_xy <- readRDS("data/auxilary-03-modis.rds")
dat_plot2 <- merge(dat_plot, dat_xy, by = "icell")

dat_plot2[, frow := fct_relevel(frow, "past")]
dat_plot2[, frow := fct_recode(frow, 
                               "past (2000-2019)" = "past",
                               "future (2071-2100)" = "future")]
dat_plot2[, fcol := fct_inorder(fcol)]

dat_plot2[is.na(rcm_short), rcm_short := "MODIS"]

out_path <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/bc_comparison/"

for(i_month in c(11, 1, 3)){
  for(i_rcm in rcm_short[rcm_short %in% unique(dat_plot2$rcm_short)]){
    
    gg <- dat_plot2[month %in% i_month & rcm_short %in% c("MODIS", i_rcm)] %>% 
      ggplot(aes(x, y, fill = value))+
      geom_raster()+
      coord_equal()+
      facet_grid(frow ~ fcol)+
      # scale_fill_distiller(palette = "Spectral", direction = 1, limits = c(0,1))+
      scale_fill_viridis_c()+
      theme_bw()+
      theme(axis.text = element_blank(),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            legend.position = "blank")
    
    # fn_out <- path(out_path, "mean",
    #                paste0(i_rcm, "_", month.abb[i_month], ".png"))
    # fn_out <- path(out_path, "q10",
    #                paste0(i_rcm, "_", month.abb[i_month], ".png"))
    fn_out <- path(out_path, "q90",
                   paste0(i_rcm, "_", month.abb[i_month], ".png"))
    
    ggsave(gg, filename = fn_out,
           width = 14, height = 6, units = "in")
    
  }
  
  
}




# some example ecdf -------------------------------------------------------

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/ecdf-01.rda")
icell_sample <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/sample-pixels-01-stratified-elev.rds")
dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")

dat_plot_ecdf <- rbindlist(use.names = T, fill = T, list(
  # clim_modis[, .(fcol = "MODIS", frow = "past",
  #                rcm = NA_character_, icell, month, value = snc)],
  ecdf_qdm[, .(f_bc = "raw", fp, 
               rcm, icell, date, value = snc)],
  ecdf_delta[, .(f_bc = "delta-change", fp = "future",
                 rcm, icell, date = as.Date(date), value = snc_future_delta)],
  ecdf_qm[, .(f_bc = "QM", fp, 
              rcm,  icell, date, value = snc_qm)],
  ecdf_qdm[, .(f_bc = "QDM", fp, 
               rcm,  icell, date, value = snc_qdm)],
  ecdf_mbc[, .(f_bc = "MBC-QDM", fp, 
               rcm,  icell, date, value = snc_qdm)]
))

rcm_short <- readRDS("data/auxilary-01-rcm-names.rds")
dat_plot_ecdf[, rcm_short := rcm_short[rcm]]

dat_plot_ecdf[, fp := fct_relevel(fp, "past")]
dat_plot_ecdf[, fp := fct_recode(fp, 
                               "past (2000-2019)" = "past",
                               "future (2071-2100)" = "future")]
dat_plot_ecdf[, f_bc := fct_inorder(f_bc)]


pdf("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/bc_comparison/ecdf-sample-icell.pdf",
    width = 14, height = 6)

for(i_icell in sort(icell_sample)){
  
  gg <- dat_plot_ecdf[icell == i_icell] %>% 
    ggplot(aes(value, colour = f_bc))+
    stat_ecdf(inherit.aes = F, data = dat_modis[icell == i_icell & !is.na(snc)], aes(snc), 
              colour = "black", linetype = "dashed", pad = F)+
    stat_ecdf(pad = F)+
    scale_color_brewer(palette = "Set1")+
    facet_grid(fp ~ rcm_short)+
    theme_bw()+
    ylim(0,1)+xlim(0,1)+
    xlab("Snow cover fraction [0-1]")+
    ylab("ECDF (empirical cumulative probability)")+
    ggtitle(paste0("cell: ", i_icell))
  
  print(gg)

}

dev.off()
