# plot time series of SNC changes
# BC data 0.11

library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(ggplot2)
library(forcats)


load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
rcm_short <- readRDS("data/auxiliary-01-rcm-names.rds")

dat_aux <- readRDS("data/auxiliary-03-modis.rds")

# alpine wide avg ---------------------------------------------------------

# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         !rcm_short %in% c("WRF331F", "HIRHAM5", "RACMO22E")] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)

# icell_common <- icell_modis # for testing

dt_modis <- clim_modis[icell %in% icell_common, 
                       .(snc = mean(snc)),
                       month]
dt_mbc <- clim_mbc[icell %in% icell_common, 
                   .(snc = mean(snc_mbc)), 
                   .(month, rcm, gcm, experiment, fp)]
dt_qdm <- clim_qdm[icell %in% icell_common, 
                   .(snc = mean(snc_qdm)), 
                   .(month, rcm, gcm, experiment, fp)]
dt_qm <- clim_qm[icell %in% icell_common,
                 .(snc = mean(snc_qm)),
                 .(month, rcm, gcm, experiment, fp)]
dt_dc <- dat_delta2[icell %in% icell_common,
                    .(snc = mean(snc_future_delta)),
                    .(month, rcm, gcm, experiment)]


dat_full <- rbindlist(list(
  modis = dt_modis,
  mbc = dt_mbc,
  qdm = dt_qdm,
  qm = dt_qm,
  dc = dt_dc
), use.names = T, fill = T, idcol = "ff")

mitmatmisc::add_month_fct(dat_full, 10)
dat_full[, rcm_short := rcm_short[rcm]]

# WRF331F not anymore on ESGF
# RACMO has so many "bad" cells
dat_full <- dat_full[is.na(rcm_short) |
                       !rcm_short %in% c("WRF331F", "RACMO22E")] 


# plot one BC -------------------------------------------------------------

# dat_full[, .N, rcm]
# dat_full[, .N, gcm]
# dat_full[ff == "mbc" & fp == "future", .N, .(rcm_short, gcm)]


dat_full[ff == "qdm" & fp == "past"] %>% 
  ggplot(aes(month_fct, snc, colour = rcm_short, linetype = gcm))+
  geom_line(aes(group = paste0(rcm,gcm)))+
  geom_line(data = dat_full[ff == "modis", .(month_fct, snc)], 
            inherit.aes = F,
            aes(month_fct, snc, group = "f"))+
  scale_color_brewer(palette = "Dark2")+
  theme_bw()+
  facet_wrap(~experiment)

# -> why lower in past?
# but better if only common cells used.
# completely gone for qm and qdm. only mbc has negative bias?!

dat_full[ff == "mbc" & fp == "future"] %>% 
  ggplot(aes(month_fct, snc, colour = rcm_short, linetype = gcm))+
  geom_line(aes(group = paste0(rcm,gcm)))+
  geom_line(data = dat_full[ff == "modis", .(month_fct, snc)], 
            inherit.aes = F,
            aes(month_fct, snc, group = "f"))+
  scale_color_brewer(palette = "Dark2")+
  theme_bw()+
  facet_wrap(~experiment)



dat_full[fp == "future" | ff == "dc"] %>% 
  ggplot(aes(month_fct, snc, colour = rcm_short, linetype = gcm))+
  geom_line(aes(group = paste0(rcm,gcm)))+
  geom_line(data = dat_full[ff == "modis", .(month_fct, snc)], 
            inherit.aes = F,
            aes(month_fct, snc, group = "f"))+
  scale_color_brewer(palette = "Dark2")+
  theme_bw()+
  facet_grid(experiment ~ ff)



