# test biascorrection
# TOO COMPUTE INTENSIVE!!!
# NEEDS 15min per icell, ie. ~ 60 days per GCM-RCM
# PLUS NEEDS MUCH RAM; up to 20GB per core?!


library(data.table)
library(magrittr)
library(eurocordexr)
library(foreach)
library(lubridate)
library(fs)
library(raster)

source("R/functions/QDM_snc.R")
source("R/functions/create_mat_neigh.R")
source("R/functions/yday_window.R")


mitmatmisc::init_parallel_ubuntu(6)


# parameters --------------------------------------------------------------

path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/01_bc_test_yday/"

# (spatial window)
k_space <- 2
# (time window)
k_time <- 45

# qdm options
qdm_opt_ratio = T
qdm_opt_trace = 0.001



# data --------------------------------------------------------------------

dat_th <- readRDS("data/auxilary-02-snow-max-threshold.rds")
dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-20d-02-snc.rds")
dat_modis[, yday := yday(date)]
dat_modis <- dat_modis[!is.na(snc)]

# sample raster (for adjacent)
rr_sample <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_20d/modis_snc_55x132_2000-02-24.tif")


# only use a sample of pixels
icell_sample <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/sample-pixels-for-test-yday.rds")



# prep rcm data 

dat_inv_past <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2000-2020/", add_files = T)
dat_inv_future <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2071-2100/", add_files = T)

# dat_loop_rcm <- dat_inv_future[, .(gcm, institute_rcm, experiment, ensemble, downscale_realisation)]
dat_loop_rcm <- dat_inv_future[experiment == "rcp85" & institute_rcm != "DMI-HIRHAM5", 
                               .(gcm, institute_rcm, experiment, ensemble, downscale_realisation)]



# main loop ---------------------------------------------------------------


# i_rcm <- 2

foreach(
  i_rcm = 1:nrow(dat_loop_rcm),
  .inorder = F
) %dopar% {
    
  dat_loop_rcm[i_rcm] %>% 
    unlist %>% 
    paste0(collapse = "_") -> fn_rcm
  fn_out_past <- path(path_out, paste0(fn_rcm, "_past"), ext = "rds")
  fn_out_fut <- path(path_out, paste0(fn_rcm, "_future"), ext = "rds")
  if(file_exists(fn_out_past) | file_exists(fn_out_fut)) return(paste0("Skipped: ", fn_rcm))
  # dat_loop_rcm[i_rcm] %>% merge(dat_inv_past, by = names(dat_loop_rcm))
  
  
  
  # past data
  dat_inv_past %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)) %>% 
    .$list_files %>% 
    .[[1]] -> files_past1
  dat_inv_past[experiment == "historical"] %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)[names(dat_loop_rcm) != "experiment"]) %>% 
    .$list_files %>% 
    .[[1]] -> files_past2
  files_past <- c(files_past1, files_past2)
  
  dat_past <- foreach(i_fn = files_past, .final = rbindlist) %do% {
    nc_grid_to_dt(i_fn, "snc", interpolate_to_standard_calendar = T)
  }
  
  # future data
  dat_inv_future %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)) %>% 
    .$list_files %>% 
    .[[1]] -> files_future
  
  dat_future <- foreach(i_fn = files_future, .final = rbindlist) %do% {
    nc_grid_to_dt(i_fn, "snc", interpolate_to_standard_calendar = T)
  }
  
  # remove cells with implausible snow (accumulation) & 
  # subset to common cells (modis & rcm)
  icell_rcm <- dat_th[rcm == dat_loop_rcm[i_rcm, institute_rcm] & snow_plausible == "yes", icell]
  icell_modis <- unique(dat_modis$icell)
  icell_yes <- icell_rcm[icell_rcm %in% icell_modis]
  
  dat_past <- dat_past[icell %in% icell_yes]
  dat_future <- dat_future[icell %in% icell_yes]
  
  # other req data
  setkey(dat_past, icell, date)
  setkey(dat_future, icell, date)
  
  dat_past[, yday := yday(date)]
  dat_future[, yday := yday(date)]
  
  dat_past[, snc := snc/100]
  dat_future[, snc := snc/100]
  
  dat_past[, snc_qdm := NA_real_]
  dat_future[, snc_qdm := NA_real_]
  
  # req loop space

  dat_icell <- data.table(icell = unique(dat_past$icell))
  
  if(k_space == 0){
    
    dat_loop_space <- copy(dat_icell)
    dat_loop_space[, icell_neigh := icell]
    
  } else {
    
    mat_neigh <- create_mat_neigh(k_space)
    dat_loop_space <- dat_icell[, 
                                .(icell_neigh = c(icell,
                                                  adjacent(rr_sample,
                                                         icell,
                                                         mat_neigh)[, 2])),
                                icell]
    
  }

  
  # req loop time
  
  dat_yday <- data.table(yday = 1:366)
  dat_loop_time <- dat_yday[, .(yday_wind = yday_window(yday, k_time)), .(yday)]
  
  l_out_hist <- list()
  l_out_fut <- list()
  
  icell_loop <- icell_sample[icell_sample %in% icell_yes]
  
  foreach(
    # i_icell = dat_icell$icell
    # i_icell = icell_sample
    i_icell = icell_loop
  ) %do% {
    
    foreach(
      i_yday = dat_yday$yday
    ) %do% {
        
      obs_hist <- dat_modis[icell %in% dat_loop_space[icell == i_icell, icell_neigh] &
                              yday %in% dat_loop_time[yday == i_yday, yday_wind]]
      
      mod_hist <- dat_past[icell %in% dat_loop_space[icell == i_icell, icell_neigh] &
                             yday %in% dat_loop_time[yday == i_yday, yday_wind]]
      
      mod_fut <- dat_future[icell %in% dat_loop_space[icell == i_icell, icell_neigh] &
                              yday %in% dat_loop_time[yday == i_yday, yday_wind]]
      
      mod_hist_eval <- dat_past[icell == i_icell & yday == i_yday]
      mod_fut_eval <- dat_future[icell == i_icell & yday == i_yday]
      
      # if(nrow(mod_hist) == 0 | nrow(mod_fut) == 0 | 
      #    nrow(mod_hist_eval) == 0 | nrow(mod_fut_eval) == 0) return("Skipped (no data)")
      
      
      l_qdm <- QDM_snc(o.c = obs_hist$snc, 
                       m.c = mod_hist$snc, 
                       m.p = mod_fut$snc,
                       ratio = qdm_opt_ratio, 
                       trace = qdm_opt_trace,
                       m.c.eval = mod_hist_eval$snc,
                       m.p.eval = mod_fut_eval$snc)
      
      # mod_hist_eval[, snc_qdm := l_qdm$mhat.c]
      # mod_fut_eval[, snc_qdm := l_qdm$mhat.p]
      # 
      # l_out_hist[[as.character(i_icell)]][[as.character(i_yday)]] <- copy(mod_hist_eval)
      # l_out_fut[[as.character(i_icell)]][[as.character(i_yday)]] <- copy(mod_fut_eval)
      
      dat_past[icell == i_icell & yday == i_yday,
               snc_qdm := l_qdm$mhat.c]
      
      dat_future[icell == i_icell & yday == i_yday,
                 snc_qdm := l_qdm$mhat.p]
      
      return(NULL)
    }
    
  }
  
  
  # save results
  # l_out_hist %>% lapply(rbindlist) %>% rbindlist -> dat_out_hist
  # l_out_fut %>% lapply(rbindlist) %>% rbindlist -> dat_out_fut
  # saveRDS(dat_out_hist, fn_out_past)
  # saveRDS(dat_out_fut, fn_out_fut)
  
  dat_past <- dat_past[icell %in% icell_sample]
  dat_future <- dat_future[icell %in% icell_sample]
  
  saveRDS(dat_past, fn_out_past)
  saveRDS(dat_future, fn_out_fut)
  
  return(paste0("Done: ", fn_rcm))
  
  
  
}

