# linearly interpolate snow cover area for 1 and 2 days missing

# ----------------------------------- #

## prereq ####

library(magrittr)
library(data.table)
library(lubridate)
library(magrittr)
library(stringr)

dt_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-01-snow-nosnow-clouds.rds")

# check na
dt_na <- dt_modis[clouds >= 0.3]
dt_na[, .N, date] %>% summary
dt_na[, .N, date] %>% .[N > 1000]
dt_na[, .N, date] %>% with(table(N))

# backup: temporal filling only:
# -> 10d max has ~240k missing obs (too much!)
# -> 20d max has ~9k missing obs (ok!)


## fill dates ####

dt_modis[clouds < 0.3, 
         snc := snow / (snow + no.snow)]

# remove pixels with 0 obs
# dt_modis[, all(is.na(snc)), icell] %>%
#   .[V1 == TRUE, icell] -> icell_to_remove
# dt_modis <- dt_modis[! icell %in% icell_to_remove ]



dt_modis[, 
         .(date = seq(min(date), max(date), by = "day")),
         .(icell)] %>% 
  merge(dt_modis[, .(icell, date, snc)], all.x = T, by = c("icell", "date")) -> dt_modis_full



## interpolate linearly for gaps of 1 and 2 days ####

setkey(dt_modis_full, icell, date)

dt_modis_full[, 
              na.rle.id := rleid(is.na(snc)), 
              .(icell)]
dt_modis_full[, 
              na.rle.id.length := .N, 
              .(icell, na.rle.id)]
dt_modis_full[, 
              snc.fill := approx(date, snc, xout = date)$y,
              .(icell)]
dt_modis_full[is.na(snc) & na.rle.id.length > 2, 
              snc.fill := NA]


## save interpolated version ####

dt_modis <- dt_modis_full[, .(icell, date, snc = snc.fill)]

saveRDS(dt_modis, file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")


## EOF ####

