# update scf_th from similar lr run

library(data.table)
library(magrittr)
library(fs)
library(raster)


# get icell data ----------------------------------------------------------

dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_impute_similar_icell/") %>% 
  lapply(readRDS) %>% 
  rbindlist -> dat_icell_minmae




# prep (old scf) data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later

rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      water = rr_water[],
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]




# add alt_diff
rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


# remove water and not available pixels
dat_scf_eval <- dat_scf[!is.na(alt) & !is.na(alt_lr) & water == 0]


# save --------------------------------------------------------------------

# icells with mae
saveRDS(dat_icell_minmae, 
        "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-02-mae-hr.rds")

# new scf
dat_new_th <- merge(dat_icell_minmae, dat_scf, 
                    by.x = "icell_min_cp_mae", by.y = "icell")

rr_scf_new <- raster(rr_scf)
rr_scf_new[dat_new_th$icell_orig] <- dat_new_th$scf_th

rr_scf_new[dat_scf_eval[!is.na(scf_th), icell]] <- dat_scf_eval[!is.na(scf_th), scf_th]

writeRaster(rr_scf_new, 
            "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v03_imputed_similar.tif",
            overwrite = T)



# mae imputed
# -> (can be calculated from imputed cell data)


# wasser_dist of next icell 
# -> (can be computed from imputed cell data)