# calculate wasserstein metric between hr altitudes in lr pixels


library(raster)
# library(stars)
# library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(scico)






# prep data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA



rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v01.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]

dat_scf_missing <- dat_scf[!is.na(alt), 
                           .(n_na = sum(is.na(scf_th)),
                             n_avail = sum(!is.na(scf_th))),
                           .(alt_f)]
setorder(dat_scf_missing, alt_f)

dat_scf_missing[, perc_na := round(n_na / (n_na + n_avail), 2)]
dat_scf_missing[, perc_avail := round(n_avail / (n_na + n_avail), 2)]

dat_scf_missing


# add alt_diff
rr_alt_lr <- aggregate(rr_alt, 
                       fact = c(37, 53), 
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]



# calc dist (~4h) ---------------------------------------------------------------



dat_scf[, ii := 1:.N, icell_lr]
dat_scf %>% 
  dcast(icell_lr ~ ii, value.var = "alt") %>% 
  as.matrix -> mat_alt_lr_icell

mat_alt_lr_icell <- mat_alt_lr_icell[, 2:ncol(mat_alt_lr_icell)]

dist_alt_lr <- proxy::dist(mat_alt_lr_icell, method = transport::wasserstein1d)

saveRDS(dist_alt_lr, 
        "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-01-wasserstein.rds")
