# create rds from rofental data

library(data.table)
library(lubridate)
library(magrittr)
library(raster)
library(foreach)
library(fs)


# approx 80min per 30yr grp and 50min per 20yr grp



path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds/"

# dirs_ds_variant <- c(
#   "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental/02_qm/",
#   "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental/03_qdm/",
#   "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental/04_mbc_qdm/"
# )

dirs_ds_variant <- c(
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental/02_qm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental/03_qdm/"
)

lapply(dirs_ds_variant, function(x){
  
  data.table(bc_var = path_file(x),
             path_rcm = dir_ls(x))
  
}) %>% rbindlist() -> dat_loop


f_read_rofental <- function(path_in){
  
  files <- dir_ls(path_in)
  
  # files <- files[1:20] # test
  
  foreach(
    i_fn = files,
    .final = rbindlist
  ) %do% {
    
    i_fn %>% 
      path_file %>% 
      path_ext_remove %>% 
      ymd -> i_date
    
    rr <- raster(i_fn)
    dt_rr <- data.table(icell = 1:ncell(rr), snc = rr[])
    dt_rr[, date := i_date]
    
    # print(path_file(i_fn)) # verbose
    
    dt_rr
  }
}


mitmatmisc::init_parallel_ubuntu(6)


foreach(
  i = 1:nrow(dat_loop),
  .inorder = F
) %dopar% {
  
  i_bc_var <- dat_loop[i, bc_var]
  i_path_rcm <- dat_loop[i, path_rcm]
  
  i_fn_out <- i_path_rcm %>% 
    path_file %>% 
    path_ext_set("rds") %>% 
    path(path_out, i_bc_var, .)
  
  if(file_exists(i_fn_out)) return(paste0("Skipped: ", i_path_rcm))
  
  dir_create(path_dir(i_fn_out))
  
  dt_rof <- f_read_rofental(path_in = i_path_rcm)
  saveRDS(dt_rof, i_fn_out)
  return(i_path_rcm)
  
}