# compare flo data to ds

# library(stars)
# library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(fs)









# flo data -------------------------------------------------------------

dir_ls("data-aux/amundsen-flo-hr/", regexp = "elev") %>% 
  lapply(function(x){
    
    path_file(x) %>% 
      path_ext_remove() %>% 
      stringr::str_split("_", simplify = T) -> x_split
    
    dat <- fread(x) %>% 
      melt(id.vars = "elev", value.name = "scf")
    
    dat[, c("gcm", "institute_rcm") := tstrsplit(variable, "_")]
    
    dat[, .(gcm, institute_rcm, 
            experiment = x_split[1,4], period = x_split[1,5], 
            elev, scf)]
    
    
  }) %>% rbindlist() -> dat_am_elev


dir_ls("data-aux/amundsen-flo-hr/", regexp = "month") %>% 
  lapply(function(x){
    
    path_file(x) %>% 
      path_ext_remove() %>% 
      stringr::str_split("_", simplify = T) -> x_split
    
    dat <- fread(x) %>% 
      melt(id.vars = "month", value.name = "scf")
    
    dat[, c("gcm", "institute_rcm") := tstrsplit(variable, "_")]
    
    dat[, .(gcm, institute_rcm, 
            experiment = x_split[1,4], period = x_split[1,5], 
            month, scf)]
    
    
  }) %>% rbindlist() -> dat_am_month



dat_am_elev[, fp := ifelse(period == "2000-2020", "past", "future")]
dat_am_month[, fp := ifelse(period == "2000-2020", "past", "future")]



# rcm data ----------------------------------------------------------------



c(
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/elev/02_qm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/elev/03_qdm/"
) %>% lapply(function(x){
  
  dir_ls(x) %>% 
    lapply(readRDS) %>% 
    rbindlist()
  
}) %>% rbindlist -> dat_ds_elev


c(
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/month/02_qm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/month/03_qdm/"
) %>% lapply(function(x){
  
  dir_ls(x) %>% 
    lapply(readRDS) %>% 
    rbindlist()
  
}) %>% rbindlist -> dat_ds_month


dat_ds_elev[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]

setorder(dat_ds_elev, bc_var, institute_rcm, gcm, experiment, fp, elev)



# subset to rcms with full region coverage --------------------------------


dat_ds_elev <- dat_ds_elev[institute_rcm %in% c("CLMcom-CCLM4-8-17", 
                                                "CLMcom-ETH-COSMO-crCLIM-v1-1",
                                                "IPSL-WRF381P")]


dat_ds_month <- dat_ds_month[institute_rcm %in% c("CLMcom-CCLM4-8-17", 
                                                  "CLMcom-ETH-COSMO-crCLIM-v1-1",
                                                  "IPSL-WRF381P")]


# elev --------------------------------------------------------------------




dat_am_elev[elev < 3500] %>% 
  ggplot(aes(scf, elev))+
  geom_path(aes(colour = institute_rcm, linetype = gcm))+
  facet_grid(period ~ experiment)+
  theme_bw()



dat_ds_elev %>% 
  ggplot(aes(scf, elev))+
  geom_path(aes(colour = institute_rcm, linetype = gcm))+
  facet_grid(fp ~ experiment + bc_var)+
  theme_bw()



dat_am_elev_diff <- dat_am_elev %>% 
  dcast(gcm + institute_rcm + experiment + elev ~ fp, value.var = "scf")

dat_am_elev_diff[, scf_diff := future - past]


dat_am_elev_diff %>% 
  ggplot(aes(scf_diff, elev))+
  geom_vline(xintercept = 0)+
  geom_path(aes(colour = institute_rcm, linetype = gcm))+
  facet_grid(. ~ experiment)+
  theme_bw()


dat_am_elev_diff_mmm <- dat_am_elev_diff[, 
                                         .(mod_mean = mean(scf_diff),
                                           mod_min = min(scf_diff),
                                           mod_max = max(scf_diff)),
                                         .(elev, experiment)]

dat_am_elev_diff_mmm %>% 
  ggplot(aes(mod_mean, elev, xmin = mod_min, xmax = mod_max))+
  geom_vline(xintercept = 0)+
  geom_ribbon(alpha = 0.5)+
  geom_path()+
  facet_grid(. ~ experiment)+
  theme_bw()




# elev: ds and am together ------------------------------------------------------

dat_plot <- rbind(
  dat_ds_elev[bc_var == "03_qdm" & experiment == "rcp85",
              .(gcm, institute_rcm, fp, elev, scf, ff = "Downscaling")],
  dat_am_elev[experiment == "rcp85" & elev > 850 & elev < 3500,
              .(gcm, institute_rcm, fp, elev, scf, ff = "AMUNDSEN")]
)

setorder(dat_plot, ff, fp, gcm, institute_rcm, elev)


dat_plot[elev < 3500] %>% 
  ggplot(aes(scf, elev, colour = ff))+
  geom_path(aes(group = paste0(ff, fp, institute_rcm, gcm)))+
  facet_wrap(~fp)+
  theme_bw()

dat_plot_mmm <- dat_plot[elev < 3500, 
                         .(mod_mean = mean(scf),
                           mod_min = min(scf),
                           mod_max = max(scf)),
                         .(ff, fp, elev)]

dat_plot_mmm %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  facet_wrap(~fp)+
  theme_bw()


dat_plot_diff <- dat_plot[elev < 3500] %>% 
  dcast(gcm + institute_rcm + elev + ff ~ fp, value.var = "scf")

dat_plot_diff[, scf_diff := future - past]
setorder(dat_plot_diff, ff, gcm, institute_rcm, elev)

dat_plot_diff %>% 
  ggplot(aes(scf_diff, elev, colour = ff))+
  geom_vline(xintercept = 0)+
  geom_path(aes(group = paste0(ff, institute_rcm, gcm)))+
  theme_bw()


# month -------------------------------------------------------------------

dat_am_month[, month_fct := mitmatmisc::month_fct(month, 10)]
dat_ds_month[, month_fct := mitmatmisc::month_fct(month, 10)]



dat_am_month %>% 
  ggplot(aes(month_fct, scf))+
  geom_line(aes(colour = institute_rcm, linetype = gcm,
                group = paste0(institute_rcm, gcm)))+
  facet_grid(experiment ~ period)+
  theme_bw()

dat_ds_month %>% 
  ggplot(aes(month_fct, scf))+
  geom_line(aes(colour = institute_rcm, linetype = gcm,
                group = paste0(institute_rcm, gcm)))+
  facet_grid(experiment ~ fp + bc_var)+
  theme_bw()


dat_am_month_diff <- dat_am_month %>% 
  dcast(gcm + institute_rcm + experiment + month_fct ~ fp, value.var = "scf")

dat_am_month_diff[, scf_diff := future - past]

dat_am_month_diff %>% 
  ggplot(aes(month_fct, scf_diff))+
  geom_hline(yintercept = 0)+
  geom_line(aes(colour = institute_rcm, linetype = gcm,
                group = paste0(institute_rcm, gcm)))+
  facet_grid(. ~ experiment)+
  theme_bw()



# month: ds and am together -----------------------------------------------


dat_plot <- rbind(
  dat_ds_month[bc_var == "03_qdm" & experiment == "rcp85",
              .(gcm, institute_rcm, fp, month_fct, scf, ff = "Downscaling")],
  dat_am_month[experiment == "rcp85",
              .(gcm, institute_rcm, fp, month_fct, scf, ff = "AMUNDSEN")]
)


dat_plot %>% 
  ggplot(aes(month_fct, scf, colour = ff))+
  geom_line(aes(group = paste0(ff, institute_rcm, gcm)))+
  facet_wrap(~fp)+
  theme_bw()

dat_plot_mmm <- dat_plot[, 
                         .(mod_mean = mean(scf),
                           mod_min = min(scf),
                           mod_max = max(scf)),
                         .(ff, fp, month_fct)]


dat_plot_mmm %>% 
  ggplot(aes(month_fct, mod_mean, ymin = mod_min, ymax = mod_max, 
             colour = ff, fill = ff, group = ff))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_line()+
  facet_wrap(~fp)+
  theme_bw()



dat_plot_diff <- dat_plot %>% 
  dcast(gcm + institute_rcm + month_fct + ff ~ fp, value.var = "scf")

dat_plot_diff[, scf_diff := future - past]


dat_plot_diff %>% 
  ggplot(aes(month_fct, scf_diff, colour = ff))+
  geom_line(aes(group = paste0(ff, institute_rcm, gcm)))+
  theme_bw()

dat_plot_diff_mmm <- dat_plot_diff[, 
                                   .(mod_mean = mean(scf_diff),
                                     mod_min = min(scf_diff),
                                     mod_max = max(scf_diff)),
                                   .(ff, month_fct)]

dat_plot_diff_mmm %>% 
  ggplot(aes(month_fct, mod_mean, ymin = mod_min, ymax = mod_max, 
           colour = ff, fill = ff, group = ff))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_line()+
  theme_bw()


# EOF ---------------------------------------------------------------------






