# compare flo data to ds

# library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(fs)
library(foreach)

rs_template_ds <- raster("data/rofental_modis_laea_flo.tif")

# overview ds ------------------------------------------------------------

c("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/02_qm/") %>% 
  lapply(function(x){
    
    data.table(bc_var = path_file(x),
               fn = dir_ls(x))
    
  }) %>% rbindlist -> dat_ds

dat_ds[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]

# overview amundsen -------------------------------------------------------


x <- "data-aux/amundsen-flo-hr/maps/"
dat_am <- data.table(fn = dir_ls(x))
dat_am[, c("zz", "gcm", "institute_rcm", "experiment", "period") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]
dat_am[, fp := ifelse(period == "2000-2020", "past", "future")]



# in common? --------------------------------------------------------------

dat_loop <- dat_am[, .(gcm, institute_rcm, experiment)] %>% 
  unique %>% 
  merge(unique(dat_ds[, .(institute_rcm, gcm, experiment)]))

# get avg data ---------------------------------------------------------------

f_read_ds <- function(fn, rs0 = rs_template_ds){
  
  rs <- raster(fn)
  rs <- crop(rs, rs0)
  rs <- rs/365.25
  rs
  
}

dat_avg <- foreach(
  i = seq_len(nrow(dat_loop)),
  .final = rbindlist
) %do% {
  
  dat_i_am <- dat_loop[i, ] %>% merge(dat_am)
  dat_i_ds <- dat_loop[i, ] %>% merge(dat_ds)
  
  rs_rof_past <- raster(dat_i_am[fp == "past", fn])
  rs_rof_future <- raster(dat_i_am[fp == "future", fn])
  
  rs_ds_past_qm <- f_read_ds(dat_i_ds[fp == "past" & bc_var == "02_qm", fn])
  rs_ds_future_qm <- f_read_ds(dat_i_ds[fp == "future" & bc_var == "02_qm", fn])
  rs_ds_past_qdm <- f_read_ds(dat_i_ds[fp == "past" & bc_var == "03_qdm", fn])
  rs_ds_future_qdm <- f_read_ds(dat_i_ds[fp == "future" & bc_var == "03_qdm", fn])
  
  rr_ds_repro <- projectRaster(is.na(rs_ds_past_qdm), rs_rof_past)
  icell_amundsen <- rr_ds_repro[] == 0

  data.table(dat_loop[i, ],
             past_amundsen = mean(rs_rof_past[icell_amundsen]),
             future_amundsen = mean(rs_rof_future[icell_amundsen]),
             past_qm = cellStats(rs_ds_past_qm, "mean"),
             future_qm = cellStats(rs_ds_future_qm, "mean"),
             past_qdm = cellStats(rs_ds_past_qdm, "mean"),
             future_qdm = cellStats(rs_ds_future_qdm, "mean"))
    
}





# plot --------------------------------------------------------------------



dat_avg %>% 
  dplyr::select(-(gcm:experiment)) %>% 
  pairs

dat_avg %>% 
  ggplot(aes(past_amundsen, past_qm))+
  geom_point(aes(colour = institute_rcm, shape = gcm))

dat_avg %>% 
  ggplot(aes(past_amundsen, past_qdm))+
  geom_point(aes(colour = institute_rcm, shape = gcm))

dat_avg %>% 
  ggplot(aes(past_qm, past_qdm))+
  geom_point(aes(colour = institute_rcm, shape = gcm))


# not comparable because of missing regions


dat_avg %>% 
  ggplot(aes(future_amundsen - past_amundsen,
             future_qm - past_qm))+
  geom_abline()+
  geom_point(aes(colour = institute_rcm, shape = gcm))


dat_avg %>% 
  ggplot(aes(future_amundsen - past_amundsen,
             future_qdm - past_qdm))+
  geom_abline()+
  geom_point(aes(colour = institute_rcm, shape = gcm))+
  theme_bw()



# clm and rca, subset to common region ------------------------------------


dat_loop2 <- dat_loop[institute_rcm %in% c("CLMcom-CCLM4-8-17", "SMHI-RCA4")]
rs_rof_template <- raster(dat_am[1, fn])

rr_ds_rca4 <- f_read_ds(dat_ds[institute_rcm == "SMHI-RCA4", fn[1]])
rr_ds_repro <- projectRaster(is.na(rr_ds_rca4), 
                             rs_rof_past)
icell_common_amundsen <- rr_ds_repro[] == 0
icell_common_ds <- !is.na(rr_ds_rca4[])

dat_avg2 <- foreach(
  i = seq_len(nrow(dat_loop2)),
  .final = rbindlist
) %do% {
  
  dat_i_am <- dat_loop2[i, ] %>% merge(dat_am)
  dat_i_ds <- dat_loop2[i, ] %>% merge(dat_ds)
  
  rs_rof_past <- raster(dat_i_am[fp == "past", fn])
  rs_rof_future <- raster(dat_i_am[fp == "future", fn])
  
  rs_ds_past_qm <- f_read_ds(dat_i_ds[fp == "past" & bc_var == "02_qm", fn])
  rs_ds_future_qm <- f_read_ds(dat_i_ds[fp == "future" & bc_var == "02_qm", fn])
  rs_ds_past_qdm <- f_read_ds(dat_i_ds[fp == "past" & bc_var == "03_qdm", fn])
  rs_ds_future_qdm <- f_read_ds(dat_i_ds[fp == "future" & bc_var == "03_qdm", fn])

  
  data.table(dat_loop2[i, ],
             past_amundsen = mean(rs_rof_past[icell_common]),
             future_amundsen = mean(rs_rof_future[icell_common]),
             past_qm = mean(rs_ds_past_qm[icell_common_ds]),
             future_qm = mean(rs_ds_future_qm[icell_common_ds]),
             past_qdm = mean(rs_ds_past_qdm[icell_common_ds]),
             future_qdm = mean(rs_ds_future_qdm[icell_common_ds]))
  
}

# plot --------------------------------------------------------------------



dat_avg2 %>% 
  dplyr::select(-(gcm:experiment)) %>% 
  pairs


dat_avg2 %>% 
  ggplot(aes(past_amundsen, past_qm))+
  geom_point(aes(colour = institute_rcm, shape = gcm))

dat_avg2 %>% 
  ggplot(aes(past_amundsen, past_qdm))+
  geom_point(aes(colour = institute_rcm, shape = gcm))

dat_avg2 %>% 
  ggplot(aes(past_qm, past_qdm))+
  geom_point(aes(colour = institute_rcm, shape = gcm))

# all the same for qm/qdm


dat_avg2 %>% 
  ggplot(aes(future_amundsen - past_amundsen,
             future_qm - past_qm))+
  geom_abline()+
  geom_point(aes(colour = institute_rcm, shape = gcm))+
  geom_smooth(method = lm, se = F)


dat_avg2 %>% 
  ggplot(aes(future_amundsen - past_amundsen,
             future_qdm - past_qdm))+
  geom_abline()+
  geom_point(aes(colour = institute_rcm, shape = gcm))+
  # geom_smooth(method = lm, se = F)+
  theme_bw()

