# compare flo data to ds

library(stars)
# library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(fs)

rr_template_ds <- raster("data/rofental_modis_laea_flo.tif")
rs_template_ds <- read_stars("data/rofental_modis_laea_flo.tif")

# overview ds ------------------------------------------------------------

dat_ds <- data.table(fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/"))
dat_ds[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]

dat_ds <- dat_ds[experiment == "rcp85" & institute_rcm %in% c(
  "CLMcom-CCLM4-8-17", 
  "CLMcom-ETH-COSMO-crCLIM-v1-1",
  "IPSL-WRF381P"
)]

# overview amundsen -------------------------------------------------------


x <- "data-aux/amundsen-flo-hr/maps/"
dat_am <- data.table(fn = dir_ls(x))
dat_am[, c("zz", "gcm", "institute_rcm", "experiment", "period") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]
dat_am[, fp := ifelse(period == "2000-2020", "past", "future")]

dat_am <- dat_am[experiment == "rcp85"]



# ensemble means ----------------------------------------------------------



rr_am_past <- stack(dat_am[fp == "past", fn])
rr_am_past_mean <- calc(rr_am_past, mean)
rr_am_past_mean

rr_am_future <- stack(dat_am[fp == "future", fn])
rr_am_future_mean <- calc(rr_am_future, mean)
rr_am_future_mean


rr_ds_past <- stack(dat_ds[fp == "past", fn])
rr_ds_past <- crop(rr_ds_past, rr_template_ds)
rr_ds_past_mean <- calc(rr_ds_past, mean)


rr_ds_future <- stack(dat_ds[fp == "future", fn])
rr_ds_future <- crop(rr_ds_future, rr_template_ds)
rr_ds_future_mean <- calc(rr_ds_future, mean)




# plot --------------------------------------------------------------------


lim_diff <- range(rr_ds_future_mean[] - rr_ds_past_mean[], 
                  (rr_am_future_mean[] - rr_am_past_mean[])*365.25,
                  na.rm  =T)

gg_am_past <- ggplot()+
  geom_stars(data = st_as_stars(rr_am_past_mean*365.25))+
  scale_fill_viridis_c("", limits = c(0,366))+
  theme_void()

gg_am_future <- ggplot()+
  geom_stars(data = st_as_stars(rr_am_future_mean*365.25))+
  scale_fill_viridis_c("", limits = c(0,366))+
  theme_void()


gg_ds_past <- ggplot()+
  geom_stars(data = st_as_stars(rr_ds_past_mean))+
  scale_fill_viridis_c("", limits = c(0,366))+
  theme_void()

gg_ds_future <- ggplot()+
  geom_stars(data = st_as_stars(rr_ds_future_mean))+
  scale_fill_viridis_c("", limits = c(0,366))+
  theme_void()

gg_am_diff <- ggplot()+
  geom_stars(data = st_as_stars((rr_am_future_mean - rr_am_past_mean)*365.25))+
  scale_fill_gradient2(limits = lim_diff)+
  theme_void()

gg_ds_diff <- ggplot()+
  geom_stars(data = st_as_stars((rr_ds_future_mean - rr_ds_past_mean)))+
  scale_fill_gradient2(limits = lim_diff)+
  theme_void()

plot_grid(gg_am_past, gg_ds_past,
          gg_am_future, gg_ds_future,
          gg_am_diff, gg_ds_diff,
          ncol = 2)

