# compare flo data to ds

# library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(foreach)
library(fs)


# flo data -------------------------------------------------------------

dir_ls("data-aux/amundsen-flo-hr/", regexp = "elev") %>% 
  lapply(function(x){
    
    path_file(x) %>% 
      path_ext_remove() %>% 
      stringr::str_split("_", simplify = T) -> x_split
    
    dat <- fread(x) %>% 
      melt(id.vars = "elev", value.name = "scf")
    
    dat[, c("gcm", "institute_rcm") := tstrsplit(variable, "_")]
    
    dat[, .(gcm, institute_rcm, 
            experiment = x_split[1,4], period = x_split[1,5], 
            elev, scf)]
    
    
  }) %>% rbindlist() -> dat_am_elev



dat_am_elev[, fp := ifelse(period == "2000-2020", "past", "future")]



# DS data rofental ----------------------------------------------------------------



dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/elev_noglacier_norca4/03_qdm/") %>% 
  lapply(readRDS) %>% 
  rbindlist -> dat_ds_elev



dat_ds_elev[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]

setorder(dat_ds_elev, bc_var, institute_rcm, gcm, experiment, fp, elev)



# alpine wide: DS ensemble ------------------------------------------------



# common icell 

dat_icell <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/common-icell.rds")
names(dat_icell)
rcms_common <- c("MODIS", "CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1",
                 "CNRM-ALADIN63", "IPSL-WRF381P", "SMHI-RCA4")
lgl_icell_common <- dat_icell[, 
                              apply(.SD, 1, all), 
                              .SDcols = rcms_common]


# glacier 


rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")
lgl_icell_noglacier <- rr_rgi[] <= 0.1

# modis data 

rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")


dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd = rr_modis[],
                        elev = rr_elev[])
# dat_modis <- dat_modis[!is.na(scd)]
# dat_modis$elev %>% qplot
# dat_modis[lgl_icell_common & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier, elev] %>% summary

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 3600, by = 100), 4200), dig.lab = 5)]

dat_modis2 <- dat_modis[lgl_icell_common & lgl_icell_noglacier, 
                        .(scd = mean(scd),
                          nn = .N),
                        .(elev_f)]

# dat_modis2 %>% ggplot(aes(scd, elev_f))+geom_point()


# qdm data 

x <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/"
dat_loop <- data.table(bc_var = path_file(x),
                       file_rcm = dir_ls(x))

dat_loop[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
           tstrsplit(path_ext_remove(path_file(file_rcm)), "_")]

# dat_loop2 <- dat_loop[institute_rcm %in% rcms_common & fp == "future"]
dat_loop2 <- dat_loop[institute_rcm %in% rcms_common]


dat_rcm2 <- foreach(
  i = seq_len(nrow(dat_loop2)),
  .final = rbindlist
) %do% {
  
  # i_rcm <- dat_loop_rcm[i, institute_rcm]
  
  i_rr <- raster(dat_loop2[i, file_rcm])
  
  dat_i <- data.table(dat_modis[, .(icell, elev, elev_f)],
                      scd = i_rr[])
  
  dat_i2 <- dat_i[lgl_icell_common & lgl_icell_noglacier, 
                  .(scd = mean(scd),
                    nn = .N),
                  .(elev_f)]
  
  dat_i3 <- dat_i2 %>% 
    cbind(dat_loop2[i, ])
  dat_i3[, file_rcm := NULL]
  dat_i3
  
  
}



# rofental modis ----------------------------------------------------------

rr_template_rofental <- raster("data/rofental_modis_laea_flo.tif")

icell_hr_rofental <- cellsFromExtent(rr_modis, rr_template_rofental)

dat_modis2_rofental <- dat_modis[lgl_icell_noglacier & icell %in% icell_hr_rofental & !is.na(scd), 
                                 .(scd = mean(scd),
                                   nn = .N),
                                 .(elev_f)]


dat_modis2_rofental_wglacier <- dat_modis[icell %in% icell_hr_rofental & !is.na(scd), 
                                          .(scd = mean(scd),
                                            nn = .N),
                                          .(elev_f)]

# alpine wide: BC ensemble ------------------------------------------------

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
dat_aux <- readRDS("data/auxiliary-03-modis.rds")


# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         !rcm_short %in% c("WRF331F", "HIRHAM5", "RACMO22E")] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)


# elev means
dat_aux[, alt_f := cut(alt, breaks = c(-10, seq(200, 2600, by = 100), 3000), dig.lab = 5)]

clim_qdm2 <- merge(clim_qdm[icell %in% icell_common], 
                   dat_aux, by = "icell")

dat_bc_elev_alps <- clim_qdm2[, 
                              .(snc = mean(snc_qdm)), 
                              .(alt_f, rcm, gcm, experiment, fp)]


# rofental: BC ensemble ------------------------------------------------

rr_template_rofental <- raster("data/rofental_modis_laea_flo.tif")
rr_template_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")

icell_rofental <- cellsFromExtent(rr_template_lr, rr_template_rofental)


dat_aux[icell %in% icell_rofental]
dat_aux[icell %in% icell_rofental] %>% with(table(alt_f))
dat_aux[icell %in% icell_rofental, qplot(alt, binwidth = 250)]
dat_bc_rofental <- clim_qdm[icell %in% icell_rofental] %>% 
  merge(dat_aux, by = "icell")

dat_bc_rofental[, alt] %>% summary
dat_bc_rofental[, alt] %>% qplot
dat_bc_rofental[, alt_f := cut(alt, breaks = seq(1750, 3000, by = 250), dig.lab = 5)]
dat_bc_rofental[, .N, alt_f]

dat_bc_rofental_elev <- dat_bc_rofental[, 
                                        .(snc = mean(snc_qdm)), 
                                        .(alt_f, rcm, gcm, experiment, fp)]

# combine all -------------------------------------------------------------

dat_plot_rcm <- rbind(
  
  dat_am_elev[, .(gcm, institute_rcm, experiment, fp, 
                  elev, 
                  scf,
                  region = "Rofental", ff = "AMUNDSEN")]
  ,
  dat_ds_elev[, .(gcm, institute_rcm, experiment, fp, 
                  elev = tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
                  scf,
                  region = "Rofental", ff = "Downscaling")]
  ,
  dat_rcm2[, .(gcm, institute_rcm, experiment, fp, 
               elev = tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
               scf = scd/365.25,
               region = "GAR", ff = "Downscaling")]
  ,
  dat_bc_rofental_elev[, .(gcm, institute_rcm = rcm, experiment, fp, 
                           elev = tstrsplit(alt_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
                           scf = snc,
                           region = "Rofental", ff = "Bias correction")]
  ,
  dat_bc_elev_alps[, .(gcm, institute_rcm = rcm, experiment, fp, 
                       elev = tstrsplit(alt_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
                       scf = snc,
                       region = "GAR", ff = "Bias correction")]
  
)







dat_plot_obs <- rbind(
  dat_modis2[, .(elev = tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans,
                 scf = scd/365.25,
                 region = "GAR")]
  ,
  dat_modis2_rofental[, .(elev = tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans,
                          scf = scd/365.25,
                          region = "Rofental")]
)





# subset to rcms with (almost) full rofental coverage --------------------------------


rcms_to_keep <- c(
  "CLMcom-CCLM4-8-17", 
  "CLMcom-ETH-COSMO-crCLIM-v1-1",
  # "CNRM-ALADIN63", # 7 lr pixels missing
  "IPSL-WRF381P",
  "SMHI-RCA4" # 2 lr pixels missing
)


dat_plot_rcm2 <- dat_plot_rcm[ff == "AMUNDSEN" | 
                                (ff != "AMUNDSEN" & institute_rcm %in% rcms_to_keep)]


dat_plot_rcm2[, .(ff, institute_rcm, gcm, experiment)] %>% 
  unique %>% 
  with(table(ff, experiment))





# model minmeanmax --------------------------------------------------------


dat_plot_mmm <- dat_plot_rcm2[,
                              .(mod_mean = mean(scf),
                                mod_min = min(scf),
                                mod_max = max(scf)),
                              .(ff, fp, experiment, region, elev)]

setorder(dat_plot_mmm, ff, fp, experiment, region, elev)


dat_plot_mmm[, ff_region := paste0(ff, " ", region)]
dat_plot_mmm[, ff_region := factor(ff_region, levels = c(
  "AMUNDSEN Rofental",
  "AMUNDSEN Rofental",
))]

dat_plot_mmm[experiment == "rcp85"] %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  geom_point(inherit.aes = F, data = dat_plot_obs,
             aes(scf, elev))+
  facet_grid(fp ~ region)+
  theme_bw()




dat_plot_mmm[region == "Rofental" & elev < 3500] %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  geom_point(inherit.aes = F, 
             data = dat_plot_obs[region == "Rofental"],
             aes(scf, elev))+
  facet_grid(fp ~ experiment)+
  theme_bw()






dat_plot_mmm[experiment == "rcp85" & ff != "Bias correction" & 
  {(region == "Rofental" & elev < 3500) | region == "GAR"}] %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff,
             alpha = region,
             group = ff_region))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  geom_point(inherit.aes = F, 
             data = dat_plot_obs,
             aes(scf, elev, alpha = region))+
  facet_grid(. ~ fp)+
  theme_bw()




dat_plot_mmm[region == "Rofental" & elev < 3500] %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  geom_point(inherit.aes = F, 
             data = dat_plot_obs[region == "Rofental"],
             aes(scf, elev))+
  facet_grid(fp ~ experiment)+
  theme_bw()



dat_plot_mmm[experiment == "rcp85"] %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff,
             alpha = region,
             group = ff_region))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  facet_wrap(~fp)+
  theme_bw()









# EOF ---------------------------------------------------------------------






