# check missing scf


library(raster)
# library(stars)
# library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(scico)






# prep data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later

rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/scf_cp50_v02_monotonic.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      water = rr_water[],
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]




# add alt_diff
rr_alt_lr <- aggregate(rr_alt, 
                       fact = c(37, 53), 
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]





# summary missing ---------------------------------------------------------

dat_scf_missing_summary <- dat_scf[!is.na(alt) & !is.na(alt_lr) & water == 0, 
                                   .(n_na = sum(is.na(scf_th)),
                                     n_avail = sum(!is.na(scf_th))),
                                   .(alt_f)]
setorder(dat_scf_missing_summary, alt_f)

dat_scf_missing_summary[, perc_na := round(n_na / (n_na + n_avail), 2)]
dat_scf_missing_summary[, perc_avail := round(n_avail / (n_na + n_avail), 2)]

dat_scf_missing_summary




# check relation of scf_th by alt and alt_diff for some altitudes ---------


dat_scf_sub <- dat_scf[!is.na(scf_th) & !is.na(alt_lr)]

dat_test <- dat_scf_sub[sample.int(.N, 10000)]
dat_test <- dat_scf_sub[alt > 3000]
dat_test <- dat_scf_sub[alt < 500] %>% .[sample.int(.N, 10000)]

dat_test %>% 
  ggplot(aes(alt_diff, alt, colour = scf_th))+
  geom_point()+
  scale_color_viridis_c()+
  theme_bw()

dat_test %>% 
  ggplot(aes(alt, scf_th))+
  geom_point()+
  theme_bw()

dat_test %>% 
  ggplot(aes(alt_diff, scf_th))+
  geom_point()+
  theme_bw()

dat_test %>% 
  ggplot(aes(alt_lr, scf_th))+
  geom_point()+
  theme_bw()

dat_test %>% 
  ggplot(aes(alt_lr, scf_th, colour = alt_diff))+
  geom_point()+
  scale_color_scico(palette = "roma", rescaler = scales::rescale_mid)+
  theme_bw()

dat_test %>% 
  ggplot(aes(alt, scf_th, colour = alt_diff))+
  geom_point()+
  scale_color_scico(palette = "roma", rescaler = scales::rescale_mid)+
  theme_bw()

dat_test %>% 
  ggplot(aes(alt_diff, scf_th, colour = alt))+
  geom_point()+
  scale_color_viridis_c()+
  theme_bw()





# gm_scf <- gam(scf_th ~ s(alt) + s(alt_diff), data = dat_test)
# gm_scf %>% summary
# plot(gm_scf)
# gm_scf2 <- gam(scf_th ~ ti(alt) + ti(alt_diff) + ti(alt, alt_diff), data = dat_test)
# # gm_scf2 <- gam(scf_th ~ ti(alt, k = 10) + 
# #                  ti(alt_diff, k = 10) + 
# #                  ti(alt, alt_diff, k = 100), data = dat_test) # k too large
# gm_scf2 %>% summary
# plot(gm_scf2)
# 
# 
# 


# calc distance hr alt between lr pixels ----------------------------------

dat_scf[, ii := 1:.N, icell_lr]
dat_scf %>% 
  dcast(icell_lr ~ ii, value.var = "alt") %>% 
  as.matrix -> mat_alt_lr_icell

mat_alt_lr_icell <- mat_alt_lr_icell[, 2:ncol(mat_alt_lr_icell)]

# dist_alt_lr <- dist(mat_alt_lr_icell) #>40min
# saveRDS(dist_alt_lr, "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/scf_cp50_v02_dist.rds")
# mat_dist <- as.matrix(dist_alt_lr)



# what metric to use?

dat1 <- dat_scf[icell_lr %in% c(1010, 1011, 2010, 2020, 3010, 3011, 4010, 4020)]
dat1 %>% 
  ggplot(aes(alt))+
  geom_histogram()+
  facet_wrap(~icell_lr + alt_lr)

mean(abs(sort(mat_alt_lr_icell[2011, ]) - sort(mat_alt_lr_icell[3010, ])))
mean(abs(sort(mat_alt_lr_icell[3011, ]) - sort(mat_alt_lr_icell[3010, ])))
mean(abs(sort(mat_alt_lr_icell[1011, ]) - sort(mat_alt_lr_icell[4011, ])))
mean(abs(sort(mat_alt_lr_icell[1011, ]) - sort(mat_alt_lr_icell[1010, ])))

ks.test(mat_alt_lr_icell[3011, ], mat_alt_lr_icell[3010, ])

dat1[, alt_f2 := cut(alt, seq(-150, 4800, by = 50), dig.lab = 5)]
dat1[, .N, .(alt_f2, icell_lr)] %>% 
  dcast(icell_lr ~ alt_f2, value.var = "N") %>% 
  as.matrix() -> mat1
mat1 <- mat1[, 2:ncol(mat1)]
mat1[is.na(mat1)] <- 0
euc_fast(mat1)




dat1[, ii := 1:.N, icell_lr]
dat1 %>% 
  dcast(icell_lr ~ ii, value.var = "alt") %>% 
  as.matrix -> mat_w1
mat_w1 <- mat_w1[, 2:ncol(mat_w1)]


# compare wasserstein to euc/man of freq bin

library(transport)
wasserstein1d(mat1[1,], mat1[2])
wasserstein1d(mat1[1,], mat1[3])
wasserstein1d(mat1[1,], mat1[3])

library(proxy)

dist_euc_bin <- proxy::dist(mat1)
dist_man_bin <- proxy::dist(mat1, "manhattan")
dist_w1 <- proxy::dist(mat_w1, method = wasserstein1d)

plot(dist_euc_bin, dist_man_bin)
plot(dist_man_bin, dist_w1)
plot(dist_euc_bin, dist_w1)
dist_euc_bin





# check some missing pixels -----------------------------------------------

dat_scf_missing <- dat_scf[!is.na(alt) & !is.na(alt_lr) & is.na(scf_th) & water == 0]

dat_scf_missing[, .N, icell_lr]
dat_scf_missing[, .N, alt_f]



# check scf_th for some missing pixels ------------------------------------

alt_f_levels <- levels(dat_scf$alt_f)

i_alt_f <- alt_f_levels[20]

# in the same pixel or similar ones


dat_scf_missing[alt_f == i_alt_f]

ii <- 3

dat_scf_missing[alt_f == i_alt_f][ii]

i_icell_lr <- dat_scf_missing[alt_f == i_alt_f][ii, icell_lr]
dat_scf_i <- dat_scf[icell_lr == i_icell_lr & alt_f == i_alt_f]
dat_scf_i$scf_th %>% qplot


dat_scf_i2 <- dat_scf[icell_lr %in% dat_scf_missing[alt_f == i_alt_f, icell_lr] &
                        alt_f == i_alt_f]

dat_scf_i2 %>% 
  ggplot(aes(scf_th))+
  geom_histogram()+
  facet_wrap(~fct_reorder(factor(icell_lr), alt_lr))

dat_scf_i2[!is.na(scf_th), .(med_th = median(scf_th),
                             mean_th = mean(scf_th),
                             nn = .N), icell_lr]

# get next best icell_lr
icell_lr_i <- unique(dat_scf_missing[alt_f == i_alt_f, icell_lr])

f_icell_dist <- function(i_icell){
  
  wasser_dist <- mat_dist[, i_icell]
  icell_min_dist <- which.min(wasser_dist[wasser_dist > 0]) %>% names %>% as.numeric
  min_dist <- mat_dist[icell_min_dist, i_icell]
  
  data.table(icell_to_check = i_icell,
             icell_with_min_dist = icell_min_dist,
             min_dist = min_dist)
}

lapply(icell_lr_i, f_icell_dist) %>% 
  rbindlist() -> dat_icell

dat_scf_i3_next <- dat_scf[icell_lr %in% dat_icell$icell_with_min_dist &
                             alt_f == i_alt_f]
dat_scf_i3_next <- dat_scf_i3_next %>% 
  merge(dat_icell, by.x = "icell_lr", by.y = "icell_with_min_dist") 


gg1 <- dat_scf_i2 %>% 
  ggplot(aes(scf_th))+
  geom_histogram()+
  facet_wrap(~icell_lr)

gg2 <- dat_scf_i3_next %>% 
  ggplot(aes(scf_th))+
  geom_histogram()+
  facet_wrap(~icell_to_check)

gg1 | gg2

d1 <- dat_scf_i2[!is.na(scf_th), 
           .(med_th = median(scf_th),
             mean_th = mean(scf_th),
             nn = .N), 
           keyby = icell_lr]

d2 <- dat_scf_i3_next[!is.na(scf_th), 
                .(med_th = median(scf_th),
                  mean_th = mean(scf_th),
                  nn = .N), 
                keyby = icell_to_check]
common_icell <- intersect(d1$icell_lr, d2$icell_to_check)
cbind(d1[icell_lr %in% common_icell], d2[icell_to_check %in% common_icell])




# check one lr cell -------------------------------------------------------

# 2730 lr icell
dat_scf_i %>% 
  ggplot(aes(alt, scf_th))+
  geom_point()

rr_alt_lr %>% plot
rr_1cell <- rr_alt_lr
rr_1cell[2730] <- 10000
plot(rr_1cell)

xy_cell <- xyFromCell(rr_1cell, 2730)
xy_res <- res(rr_alt_lr)
ext <- extent(c(xmin = xy_cell[1, "x"] - xy_res[1]/2,
         xmax = xy_cell[1, "x"] + xy_res[1]/2, 
         ymin = xy_cell[1, "y"] - xy_res[2]/2,
         ymax = xy_cell[1, "y"] + xy_res[2]/2))
# ext <- extent(c(xmin = xy_cell[1, "x"],
#                 xmax = xy_cell[1, "x"] + xy_res[1], 
#                 ymin = xy_cell[1, "y"],
#                 ymax = xy_cell[1, "y"] + xy_res[2]))
rr_alt_1cell <- crop(rr_alt, ext)

# mapview(rr_1cell)+mapview(rr_alt_1cell)
plot(rr_1cell)
plot(crop(rr_1cell, extent(4400000, 4500000, 2600000, 2700000)))
plot(rr_alt_1cell, add = T)

rr_scf_1cell <- crop(rr_scf, rr_alt_1cell)
plot(rr_scf_1cell)

# mapview(rr_scf_1cell)+mapview(rr_alt_1cell)

dat_scf_i

rr_scf_1cell_check <- rr_scf_1cell
rr_scf_1cell_check[!(rr_alt_1cell > 1800 & rr_alt_1cell <= 1900)] <- NA
rr_scf_1cell_check %>% plot

# mapview(rr_scf_1cell)+mapview(rr_scf_1cell_check)

# can rr_cp be read in memory? --------------------------------------------


# mat_cp_full <- raster::as.array(rr_cp) # x*y*l
mat_cp_full <- raster::as.matrix(rr_cp) # icell*l

mat_cp_full[mat_cp_full < 0] <- NA
colnames(mat_cp_full) <- band_lbl
rownames(mat_cp_full) <- 1:ncell(rr_cp)



# fill based on cp diff ---------------------------------------------------------

source("R/functions/cp_rr2dt.R")
rr_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins_monotonic.tif")
band_lbl <- levels(cut(0:100/100, 
                       breaks = c(0, 0.001, 1:19/20, 0.999, 1), 
                       include.lowest = T))
  
dat_scf_i_lr <- dat_scf[icell_lr == 2730]
dat_scf_i_lr[is.na(scf_th)]
dat_scf_i_lr[is.na(scf_th) & alt_f == i_alt_f]
dat_scf_i_lr[is.na(scf_th), .N, keyby = alt_f]

dat_cp_i <- cp_rr2dt(dat_scf_i_lr$icell)
dat_cp_i
dat_cp_i[icell %in% dat_scf_i_lr[is.na(scf_th) & alt_f == i_alt_f, icell]] %>% 
  ggplot(aes(scf_middle, value))+
  geom_point()+
  facet_wrap(~icell)


# dat_cp_i[icell %in% dat_scf_i_lr[is.na(scf_th) & alt_f == "(800,900]", icell]] %>%
# dat_cp_i[icell %in% dat_scf_i_lr[is.na(scf_th) & alt_f == "(1500,1600]", icell]] %>% 
dat_cp_i[icell %in% dat_scf_i_lr[!is.na(scf_th) & alt_f == "(800,900]", sample(icell, 30)]] %>%
  ggplot(aes(scf_middle, value))+
  geom_point()+
  facet_wrap(~icell)



dat_cp_i

# find minimum distance cp
mat_cp_icell_lr <- cp_rr2mat(dat_scf_i_lr$icell)

# similar icell_lr
icell_min_dist(2730)
mat_cp_icell_lr2 <- cp_rr2mat(dat_scf[icell_lr == 3483 & !is.na(scf_th), icell])



icell_lr_i_missing <- dat_scf_i_lr[is.na(scf_th), icell]

mat_cp_dist <- euc_fast(mat_cp_icell_lr, na.rm = T) 
mat_cp_dist %>% is.na %>% sum
mat_cp_dist %>% dim %>% prod # all NA?!
# mat_cp_dist_missing <- mat_cp_dist[as.character(icell_lr_i_missing), ]


mat_cp_icell_lr %>% is.na %>% rowSums %>% summary
mat_cp_icell_lr2 %>% is.na %>% rowSums %>% summary



eps = 0.001
n_min_cp = 5

lapply(as.character(icell_lr_i_missing), function(x){
  
  x_cp <- mat_cp_icell_lr[x, ]
  x_cp[x_cp < eps | (1-x_cp) < eps] <- NA
  if(sum(is.na(x_cp)) < n_min_cp) return(NULL)
  
  mat_abs_diff <- abs(sweep(mat_cp_icell_lr2, 2, x_cp, "-"))
  icell_n <- length(x_cp) - (mat_abs_diff %>% is.na %>% rowSums)
  
  mae <- rowMeans(mat_abs_diff, na.rm = T)
  mae[icell_n < n_min_cp] <- NA
  
  icell_min <- names(which.min(mae[mae > 0]))
  data.table(icell_min_cp_mae = icell_min,
             min_mae = mae[icell_min])
})




lapply(as.character(icell_lr_i_missing), function(x){
  
  x_cp <- mat_cp_icell_lr[x, ]
  cp_mae(x_cp, mat_cp_icell_lr2)
  
}) %>% rbindlist -> dat_next_icell

dat_next_icell[, icell_orig := icell_lr_i_missing]

dat_next_icell %>% 
  merge(dat_scf, by.x = "icell_min_cp_mae", by.y = "icell") -> dat_next_icell_th


# check scf_th map 
rr_scf_1cell <- crop(rr_scf, rr_alt_1cell)

rr_scf_new <- rr_scf
rr_scf_new[dat_next_icell_th$icell_orig] <- dat_next_icell_th$scf_th
rr_scf_1cell_new <- crop(rr_scf_new, rr_alt_1cell)


rr_scf_new <- raster(rr_scf)
rr_scf_new[dat_next_icell_th$icell_orig] <- dat_next_icell_th$min_mae
rr_scf_1cell_mae <- crop(rr_scf_new, rr_alt_1cell)

plot(rr_scf_1cell_mae)
plot(rr_scf_1cell_new)
plot(rr_scf_1cell)
plot(stack(rr_scf_1cell, rr_scf_1cell_new), zlim = c(0,1))

rr_scf_
rr_scf_1cell_new_large <- rr_scf_1cell_new
plot(stack(rr_scf_1cell, rr_scf_1cell_new, rr_scf_1cell_new_large), zlim = c(0,1))

icell_min_dist(2730, 40)
rr_next <- rr_alt_lr
rr_next[2730] <- 10000
rr_next[icell_min_dist(2730, 40)$icell_with_min_dist] <- 8000
plot(rr_next)


# increase points to search for
icell_min_dist(2730, 10)
icell_to_check <- dat_scf[!is.na(scf_th) & 
                            icell_lr %in% icell_min_dist(2730, 10)$icell_with_min_dist, 
                          icell]
mat_cp_icell_lr_large <- mat_cp_full[icell_to_check, ]

lapply(as.character(icell_lr_i_missing), function(x){
  
  x_cp <- mat_cp_icell_lr[x, ]
  cat(x, "\n")
  cp_mae(x_cp, mat_cp_icell_lr_large)
  
}) %>% rbindlist -> dat_next_icell

dat_next_icell[, icell_orig := icell_lr_i_missing]

dat_next_icell %>% 
  merge(dat_scf, by.x = "icell_min_cp_mae", by.y = "icell") -> dat_next_icell_th



# increase lr_cells but limit to same alt_f
# takes MUCH longer (because getting cp from raster is slow)
# if full mat cp is read in, then easy!?
icell_lr_similar <- icell_min_dist(2730, 50)$icell_with_min_dist

dat_scf_sub <- dat_scf[!is.na(scf_th) & 
                         icell_lr %in% icell_lr_similar]

lapply(as.character(icell_lr_i_missing), function(x){
  
  x_cp <- mat_cp_icell_lr[x, ]

  icell_to_check <- dat_scf_sub[alt_f == dat_scf[as.integer(x), alt_f], 
                                icell]
  mat_cp_icell_lr_large <- mat_cp_full[icell_to_check, ]
  cat(x, "\n")
  cp_mae(x_cp, mat_cp_icell_lr_large)
  
}) %>% rbindlist -> dat_next_icell

dat_next_icell[, icell_orig := icell_lr_i_missing]

dat_next_icell %>% 
  merge(dat_scf, by.x = "icell_min_cp_mae", by.y = "icell") -> dat_next_icell_th


# check cp curves

dat_check_cp <- copy(dat_next_icell_th)
dat_check_cp[, icell4plot := icell_orig]
zz_cp <- cp_rr2dt(dat_check_cp[, c(icell_min_cp_mae, icell_orig)])

dat_check_cp %>% 
  melt(measure.vars = c("icell_min_cp_mae", "icell_orig"),
       id.vars = c("min_mae", "icell4plot", "scf_th"),
       value.name = "icell",
       variable.name = "zz") %>% 
  merge(zz_cp, by = "icell", allow.cartesian = T) -> dat_plot2

dat_plot2[, min_mae_plot := round(min_mae, 3)]

dat_plot2[min_mae > 0.05, .N, icell4plot]
dat_plot2[min_mae > 0.05] %>% 
  ggplot(aes(scf_middle, value, colour = zz))+
  geom_point()+
  facet_wrap(~icell4plot + min_mae_plot)+
  theme_bw()

dat_plot2[min_mae < 0.015, .N, icell4plot]
dat_plot2[min_mae < 0.015] %>% 
  ggplot(aes(scf_middle, value, colour = zz))+
  geom_point()+
  facet_wrap(~icell4plot + min_mae_plot)+
  theme_bw()


# works great!!
# check for other alt (~ 300 m, and 3000, 3700)


# fill based on cp diff (low elev check) ---------------------------------------------------------


alt_f_levels <- levels(dat_scf$alt_f)
i_alt_f <- alt_f_levels[4]



dat_scf_missing[alt_f == i_alt_f]
dat_scf_missing[alt_f == i_alt_f, .N, icell_lr]

icell_lr_todo <- 258

dat_scf_i_lr <- dat_scf[icell_lr == icell_lr_todo]
dat_scf_i_lr[is.na(scf_th)]
dat_scf_i_lr[is.na(scf_th) & alt_f == i_alt_f]
dat_scf_i_lr[is.na(scf_th), .N, keyby = alt_f]


icell_lr_i_missing <- dat_scf_i_lr[is.na(scf_th), icell]


icell_min_dist(icell_lr_todo, 50)
rr_next <- rr_alt_lr
rr_next[icell_lr_todo] <- 10000
rr_next[icell_min_dist(icell_lr_todo, 50)$icell_with_min_dist] <- 8000
plot(rr_next)


mat_cp_icell_next <- mat_cp_full[dat_scf[icell_lr == 380 & !is.na(scf_th), icell], ]


lapply(as.character(icell_lr_i_missing), function(x){
  
  x_cp <- mat_cp_full[x, ]
  cat(x, "\n")
  cp_mae(x, x_cp, mat_cp_icell_next)
  
}) %>% rbindlist -> dat_next_icell

# dat_next_icell[, icell_orig := icell_lr_i_missing]

dat_next_icell %>% 
  merge(dat_scf, by.x = "icell_min_cp_mae", by.y = "icell") -> dat_next_icell_th


# check scf_th map 

xy_cell <- xyFromCell(rr_1cell, icell_lr_todo)
xy_res <- res(rr_alt_lr)
ext <- extent(c(xmin = xy_cell[1, "x"] - xy_res[1]/2,
                xmax = xy_cell[1, "x"] + xy_res[1]/2, 
                ymin = xy_cell[1, "y"] - xy_res[2]/2,
                ymax = xy_cell[1, "y"] + xy_res[2]/2))
rr_alt_1cell <- crop(rr_alt, ext)


rr_scf_new <- rr_scf
rr_scf_new[dat_next_icell_th$icell_orig] <- dat_next_icell_th$scf_th
rr_scf_1cell_new <- crop(rr_scf_new, rr_alt_1cell)


rr_scf_new <- raster(rr_scf)
rr_scf_new[dat_next_icell_th$icell_orig] <- dat_next_icell_th$min_mae
rr_scf_1cell_mae <- crop(rr_scf_new, rr_alt_1cell)

plot(rr_scf_1cell_mae)
plot(rr_scf_1cell_new)
plot(rr_scf_1cell)
plot(stack(rr_scf_1cell, rr_scf_1cell_new), zlim = c(0,1))

rr_scf_1cell_new_same_icell <- rr_scf_1cell_new


plot(stack(rr_scf_1cell, rr_scf_1cell_new_same_icell, rr_scf_1cell_new), zlim = c(0,1))
plot(stack(rr_scf_1cell_new_same_icell, rr_scf_1cell_new, rr_scf_1cell), zlim = c(0,1))


# increase lr_cells but limit to same alt_f
# takes MUCH longer (because getting cp from raster is slow)
# if full mat cp is read in, then easy!?
icell_lr_similar <- icell_min_dist(icell_lr_todo, 50)$icell_with_min_dist

dat_scf_sub <- dat_scf[!is.na(scf_th) & 
                         icell_lr %in% icell_lr_similar]

lapply(as.character(icell_lr_i_missing), function(x){
  
  x_cp <- mat_cp_full[x, ]
  
  icell_to_check <- dat_scf_sub[alt_f == dat_scf[as.integer(x), alt_f], 
                                icell]
  mat_cp_icell_lr_large <- mat_cp_full[icell_to_check, ]
  cat(x, "\n")
  cp_mae(x, x_cp, mat_cp_icell_lr_large)
  
}) %>% rbindlist -> dat_next_icell

# dat_next_icell[, icell_orig := icell_lr_i_missing]

dat_next_icell %>% 
  merge(dat_scf, by.x = "icell_min_cp_mae", by.y = "icell") -> dat_next_icell_th


# check cp curves

dat_check_cp <- copy(dat_next_icell_th)
dat_check_cp[, icell4plot := icell_orig]
zz_cp <- cp_rr2dt(dat_check_cp[, c(icell_min_cp_mae, icell_orig)])

dat_check_cp %>% 
  melt(measure.vars = c("icell_min_cp_mae", "icell_orig"),
       id.vars = c("min_mae", "icell4plot", "scf_th"),
       value.name = "icell",
       variable.name = "zz") %>% 
  merge(zz_cp, by = "icell", allow.cartesian = T) -> dat_plot2

dat_plot2[, min_mae_plot := round(min_mae, 3)]
dat_plot2$min_mae %>% summary

dat_plot2[min_mae > 0.015, .N, icell4plot]
dat_plot2[min_mae > 0.015] %>% 
  ggplot(aes(scf_middle, value, colour = zz))+
  geom_point()+
  facet_wrap(~icell4plot + min_mae_plot)+
  theme_bw()

dat_plot2[min_mae < 0.003, .N, icell4plot]
dat_plot2[min_mae < 0.003] %>% 
  ggplot(aes(scf_middle, value, colour = zz))+
  geom_point()+
  facet_wrap(~icell4plot + min_mae_plot)+
  theme_bw()



# fill based on cp diff (high elev check) ---------------------------------------------------------


alt_f_levels <- levels(dat_scf$alt_f)
i_alt_f <- alt_f_levels[40]



dat_scf_missing[alt_f == i_alt_f]
dat_scf_missing[alt_f == i_alt_f, .N, icell_lr]

icell_lr_todo <- 3627 # ~ 4000m
icell_lr_todo <- 2208 # ~ 3000m
icell_lr_todo <- 373 # ~ 300m

dat_scf_i_lr <- dat_scf[icell_lr == icell_lr_todo]
dat_scf_i_lr[is.na(scf_th)]
dat_scf_i_lr[is.na(scf_th) & alt_f == i_alt_f]
dat_scf_i_lr[is.na(scf_th), .N, keyby = alt_f]


icell_lr_i_missing <- dat_scf_i_lr[is.na(scf_th), icell]


icell_min_dist(icell_lr_todo, 50)
icell_lr_next <- icell_min_dist(icell_lr_todo, 50)[1, icell_with_min_dist]
rr_next <- rr_alt_lr
rr_next[icell_lr_todo] <- 20000
rr_next[icell_min_dist(icell_lr_todo, 50)$icell_with_min_dist] <- 10000
plot(rr_next)


mat_cp_icell_next <- mat_cp_full[dat_scf[icell_lr == icell_lr_next & !is.na(scf_th), icell], ]


lapply(as.character(icell_lr_i_missing), function(x){
  
  x_cp <- mat_cp_full[x, ]
  cat(x, "\n")
  cp_mae(x, x_cp, mat_cp = mat_cp_icell_next)
  
}) %>% rbindlist -> dat_next_icell

# dat_next_icell[, icell_orig := icell_lr_i_missing]

dat_next_icell %>% 
  merge(dat_scf, by.x = "icell_min_cp_mae", by.y = "icell") -> dat_next_icell_th


# check scf_th map 

xy_cell <- xyFromCell(rr_1cell, icell_lr_todo)
xy_res <- res(rr_alt_lr)
ext <- extent(c(xmin = xy_cell[1, "x"] - xy_res[1]/2,
                xmax = xy_cell[1, "x"] + xy_res[1]/2, 
                ymin = xy_cell[1, "y"] - xy_res[2]/2,
                ymax = xy_cell[1, "y"] + xy_res[2]/2))
rr_alt_1cell <- crop(rr_alt, ext)
plot(rr_alt_1cell)
rr_scf_1cell <- crop(rr_scf, ext)


rr_scf_new <- rr_scf
rr_scf_new[dat_next_icell_th$icell_orig] <- dat_next_icell_th$scf_th
rr_scf_1cell_new <- crop(rr_scf_new, rr_alt_1cell)


rr_scf_new <- raster(rr_scf)
rr_scf_new[dat_next_icell_th$icell_orig] <- dat_next_icell_th$min_mae
rr_scf_1cell_mae <- crop(rr_scf_new, rr_alt_1cell)

plot(rr_scf_1cell_mae)
plot(rr_scf_1cell_new)
plot(rr_scf_1cell)
plot(stack(rr_scf_1cell, rr_scf_1cell_new), zlim = c(0,1))

rr_scf_1cell_new_same_icell <- rr_scf_1cell_new


plot(stack(rr_scf_1cell, rr_scf_1cell_new_same_icell, rr_scf_1cell_new), zlim = c(0,1))
plot(stack(rr_scf_1cell_new_same_icell, rr_scf_1cell_new, rr_scf_1cell), zlim = c(0,1))


# increase lr_cells but limit to same alt_f
# takes MUCH longer (because getting cp from raster is slow)
# if full mat cp is read in, then easy!?
icell_lr_similar <- icell_min_dist(icell_lr_todo, 50)$icell_with_min_dist

dat_scf_sub <- dat_scf[!is.na(scf_th) & 
                         icell_lr %in% icell_lr_similar]

lapply(as.character(icell_lr_i_missing), function(x){
  
  x_cp <- mat_cp_full[x, ]
  
  icell_to_check <- dat_scf_sub[alt_f == dat_scf[as.integer(x), alt_f], 
                                icell]
  mat_cp_icell_lr_large <- mat_cp_full[icell_to_check, , drop = F]
  cat(x, "\n")
  cp_mae(x, x_cp, mat_cp_icell_lr_large)
  
}) %>% rbindlist -> dat_next_icell

# dat_next_icell[, icell_orig := icell_lr_i_missing]

dat_next_icell %>% 
  merge(dat_scf, by.x = "icell_min_cp_mae", by.y = "icell") -> dat_next_icell_th


# check cp curves

dat_check_cp <- copy(dat_next_icell_th)
dat_check_cp[, icell4plot := icell_orig]
zz_cp <- cp_rr2dt(dat_check_cp[, unique(c(icell_min_cp_mae, icell_orig))])

dat_check_cp %>% 
  melt(measure.vars = c("icell_min_cp_mae", "icell_orig"),
       id.vars = c("min_mae", "icell4plot", "scf_th"),
       value.name = "icell",
       variable.name = "zz") %>% 
  merge(zz_cp, by = "icell", allow.cartesian = T) -> dat_plot2

dat_plot2[, min_mae_plot := round(min_mae, 3)]
dat_plot2$min_mae %>% summary

dat_plot2[min_mae > 0.05, .N, icell4plot]
dat_plot2[min_mae > 0.05] %>% 
  ggplot(aes(scf_middle, value, colour = zz, shape = zz))+
  geom_point()+
  scale_shape_manual(values = c(3, 4))+
  facet_wrap(~icell4plot + min_mae_plot)+
  theme_bw()

dat_plot2[min_mae < 0.008, .N, icell4plot]
dat_plot2[min_mae < 0.008] %>% 
  ggplot(aes(scf_middle, value, colour = zz, shape = zz))+
  geom_point()+
  scale_shape_manual(values = c(3, 4))+
  facet_wrap(~icell4plot + min_mae_plot)+
  theme_bw()


# make monotonic cp -------------------------------------------------------

dat_mon <- dat_plot2[icell4plot == 4306023]
x <- dat_mon[zz == "icell_orig", value]

source("R/functions/longest_subseq.R")
x_i <- longest_subseq(x)
x_sub <- longest_subseq_na(x)

plot(x)
points(x_i, x[x_i], pch = 4)

dat_plot2[, value_mon2 := monotonic_cp_buffer(value), .(icell4plot, zz)]

dat_plot2[min_mae > 0.03] %>%
# dat_plot2[min_mae < 0.0020] %>%
  ggplot(aes(scf_middle, value, colour = zz))+
  geom_point()+
  geom_point(aes(y = value_mon2), shape = 3, colour = "black")+
  facet_wrap(~icell4plot + zz + icell)+
  theme_bw()

# chekc some

dat_plot2[icell4plot == 4394236]
dat_plot2[icell == 10268566]
x <- dat_plot2[icell4plot == 4394236 & zz == "icell_min_cp_mae", value]
x <- dat_plot2[icell4plot == 4306021 & zz == "icell_min_cp_mae", value]
x <- dat_plot2[icell4plot == 4168775 & zz == "icell_orig", value]
# x <- dat_plot2[icell == 4790623 & zz == "icell_orig", value]
x
x_sub <- monotonic_cp(x)
x_sub <- monotonic_cp_buffer(x)
plot(x)
points(x_sub, pch = 4)



# EOF ---------------------------------------------------------------------


