# compare scf_th across levels
# - from cp
# - impute cp
# - impute alt



library(raster)
# library(stars)
# library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(scico)


rr_scf1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
rr_scf2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v03_imputed_similar.tif")
rr_scf3 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif")




# prep data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later



dat_scf <- data.table(icell = 1:ncell(rr_scf1),
                      water = rr_water[],
                      scf1 = rr_scf1[],
                      scf2 = rr_scf2[],
                      scf3= rr_scf3[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]




# add alt_diff
rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


# remove water and not available pixels
dat_scf_eval <- dat_scf[!is.na(alt) & !is.na(alt_lr) & water == 0]



# maps --------------------------------------------------------------------


plot(rr_scf1)
plot(rr_scf2)
plot(rr_scf3)

rr_scf3_sub <- crop(rr_scf3, extent(c(xmin = 3847098, xmax = 4000000,
                                      ymin = 2500000, ymax = 2800000)))
plot(rr_scf3_sub)

rr_alt_sub <- crop(rr_alt, rr_scf3_sub)
plot(rr_alt_sub)

mapview(rr_scf3_sub)+mapview(rr_alt_sub)


rr_scf3_new <- rr_scf3
rr_scf3_new[!is.na(rr_scf2)] <- NA
plot(rr_scf3_new)

# compare n missing -------------------------------------------------------

dat_scf_eval_missing <- dat_scf_eval[, 
                                     .(n_na_1 = sum(is.na(scf1)),
                                       n_avail_1 = sum(!is.na(scf1)),
                                       n_na_2 = sum(is.na(scf2)),
                                       n_avail_2 = sum(!is.na(scf2)),
                                       n_na_3 = sum(is.na(scf3)),
                                       n_avail_3 = sum(!is.na(scf3))),
                                     .(alt_f)]

setorder(dat_scf_eval_missing, alt_f)

n_vec <- dat_scf_eval_missing[, n_na_1 + n_avail_1]
dat_scf_eval_missing_perc <- dat_scf_eval_missing[, 
                                                  lapply(.SD, function(x) x / n_vec), 
                                                  .SDcols = names(dat_scf_eval_missing)[-1]]
dat_scf_eval_missing_perc[, alt_f := dat_scf_eval_missing$alt_f]

dat_scf_eval_missing_perc %>% 
  melt(id.vars = "alt_f") %>% 
  ggplot(aes(alt_f, value))+
  geom_col()+
  facet_wrap(~variable, ncol = 2)



# rofental ----------------------------------------------------------------

rr_rofental <- raster("data/rofental_modis_laea.tif")

rrs_rof <- stack(crop(rr_scf1, rr_rofental),
                 crop(rr_scf2, rr_rofental),
                 crop(rr_scf3, rr_rofental))
plot(rrs_rof, zlim = c(-0.5,1))



# compare to only next icell ----------------------------------------------

dat_dist1 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-02-mae-hr.rds")
dat_dist2 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_impute_only_next_icell/dist-02-mae-hr.rds")

dat_zz <-merge(dat_dist1, 
      dat_dist2[, .(icell_orig, icell_min_cp_mae2 = icell_min_cp_mae, min_mae2 = min_mae)],
      all = T)

dat_zz %>% summary
dat_zz$min_mae %>% qplot
dat_zz$min_mae2 %>% qplot

dat_zz %>% 
  melt(measure.vars = c("min_mae", "min_mae2")) %>% 
  ggplot(aes(value))+
  geom_histogram()+
  facet_wrap(~variable)


dat_zz %>% 
  melt(measure.vars = c("min_mae", "min_mae2")) %>% 
  ggplot(aes(value, colour = variable))+
  geom_freqpoly(bins = 50)+
  xlim(0, 0.02)

dat_zz[, qplot(min_mae2 - min_mae)]
dat_zz[, qplot(min_mae2 - min_mae)+xlim(NA, 0.01)]
dat_zz[, summary(min_mae2 - min_mae)]


rr_scf3b <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_impute_only_next_icell/scf_cp50_v04_imputed_alt.tif")
rr_scf_check <- rr_scf3b - rr_scf3
plot(rr_scf_check)
rr_scf_check[rr_scf_check == 0] <- NA
plot(rr_scf_check)
