# compare ds of past with modis

library(data.table)
library(raster)
library(magrittr)
library(ggplot2)
library(fs)

source("R/functions/calc_clim_ds.R")


# check past --------------------------------------------------------------

rr_clim_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
plot(rr_clim_modis)

fns_past <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/")
rrs <- stack(fns_past[10:21])
# rrs_sum <- stackApply()
rrs_sum <- calc(rrs, sum)

dt_fns <- data.table(fn = fns_past)
dt_fns[, year_month := fns_past %>% 
         path_file %>% 
         path_ext_remove]
dt_fns[, c("year", "month") := tstrsplit(year_month, "-", type.convert = T)]
mitmatmisc::add_hydro_year(dt_fns)
dt_fns[, .N, hydro_year]
dt_fns_hydro <- dt_fns[hydro_year %in% 2001:2020]

# too large
# rrs_all <- stack(dt_fns_hydro$fn)
# rrs_all_yearSCD <- stackApply(rrs_all, dt_fns_hydro$month, sum)
# rrs_all_clim <- calc(rrs_all_yearSCD, mean)


# try year by year
path_temp_proc <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/temp_r_proc/"
dir_create(path_temp_proc)

for(i_hy in unique(dt_fns_hydro$hydro_year)){

  fn_out <- path(path_temp_proc, i_hy, ext = "tif")
  fns_i <- dt_fns_hydro[hydro_year == i_hy, fn]
  rrs_i <- stack(fns_i)
  rrs_i_out <- calc(rrs_i, sum)
  writeRaster(rrs_i_out, fn_out)

}

dir_ls(path_temp_proc)
rrs <- stack(dir_ls(path_temp_proc))
rrs_clim <- calc(rrs, mean)
rrs_clim[rrs_clim < 0] <- NA
plot(rrs_clim)
writeRaster(rrs_clim, "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/clm_past_clim.tif")

dir_delete(path_temp_proc)

plot(rrs_clim, rr_clim_modis)
abline(0,1,col="red")

rrs_clim_py <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/zz_py_temp/test-ds-cp20/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")

plot(rrs_clim_py, rr_clim_modis)
abline(0,1,col="red")

plot(rrs_clim, rrs_clim_py)
abline(0,1,col="red")

# check future ------------------------------------------------------------

paths_todo <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp10/")[1:3]
paths_todo <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp10/", 
                     type = "directory")[3]

for(i in seq_along(paths_todo)){
  
  fn_out <- paths_todo[i] %>% 
    path_file %>% 
    path_ext_set("tif") %>% 
    path("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp10/", .)
  
  calc_clim_ds(paths_todo[i], fn_out)
  
}


rr1_f <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp10/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.tif")
rr1_p <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp10/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")
rr2_f <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp10/CLMcom-CCLM4-8-17_ICHEC-EC-EARTH_rcp26_r12i1p1_v1_future.tif")

plot(rr1_f)
plot(rr1_p)
plot(rr2_f)


plot(rr1_f, rr_clim_modis)
abline(0,1,col="red")

plot(rr1_p, rr_clim_modis)
abline(0,1,col="red")

plot(rr1_f, rr_clim_modis)
abline(0,1,col="red")

dat_cp10 <- data.table(scd_modis = rr_clim_modis[],
                       scd_past = rr1_p[],
                       scd_fut85 = rr1_f[],
                       scd_fut26 = rr2_f[])

na.omit(dat_cp10) %>% 
  ggplot(aes(scd_modis, scd_past))+
  geom_bin2d()+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")

na.omit(dat_cp10) %>% 
  ggplot(aes(scd_modis, scd_fut85))+
  geom_bin2d()+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")

na.omit(dat_cp10) %>% 
  ggplot(aes(scd_modis, scd_fut26))+
  geom_bin2d()+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")


na.omit(dat_cp10) %>% 
  .[, .(bias_past = mean(scd_past - scd_modis),
        bias_fut85 = mean(scd_fut85 - scd_modis),
        bias_fut26 = mean(scd_fut26 - scd_modis),
        mae_past = mean(abs(scd_past - scd_modis)),
        mae_fut85 = mean(abs(scd_fut85 - scd_modis)),
        mae_fut26 = mean(abs(scd_fut26 - scd_modis)))] -> dt_cp10_summ
  

# compare 10bins to 20bins -------------------------------------------------------

paths_todo <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20/", 
                     type = "directory")

for(i in seq_along(paths_todo)){
  
  fn_out <- paths_todo[i] %>% 
    path_file %>% 
    path_ext_set("tif") %>% 
    path("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20/", .)
  
  calc_clim_ds(paths_todo[i], fn_out)
  
}




rr1_f_20 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.tif")
rr1_p_20 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")
rr2_f_20 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20/CLMcom-CCLM4-8-17_ICHEC-EC-EARTH_rcp26_r12i1p1_v1_future.tif")


plot(rr1_p_20)
plot(rr1_p)
plot(rr_clim_modis)

plot(rr1_f_20)
plot(rr1_p_20)
plot(rr2_f_20)


dat_cp20 <- data.table(scd_modis = rr_clim_modis[],
                       scd_past = rr1_p_20[],
                       scd_fut85 = rr1_f_20[],
                       scd_fut26 = rr2_f_20[])

na.omit(dat_cp20) %>% 
  ggplot(aes(scd_modis, scd_past))+
  geom_bin2d()+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")

na.omit(dat_cp20) %>% 
  ggplot(aes(scd_modis, scd_fut85))+
  geom_bin2d()+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")

na.omit(dat_cp20) %>% 
  ggplot(aes(scd_modis, scd_fut26))+
  geom_bin2d()+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")


na.omit(dat_cp20) %>% 
  .[, .(bias_past = mean(scd_past - scd_modis),
        bias_fut85 = mean(scd_fut85 - scd_modis),
        bias_fut26 = mean(scd_fut26 - scd_modis),
        mae_past = mean(abs(scd_past - scd_modis)),
        mae_fut85 = mean(abs(scd_fut85 - scd_modis)),
        mae_fut26 = mean(abs(scd_fut26 - scd_modis)))] -> dt_cp20_summ

rbind(dt_cp20_summ,
      dt_cp10_summ)


# check minus 1 -----------------------------------------------------------


rr0 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/2000-01.tif")
rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20-minus1/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/2000-01.tif")


plot(rr0)
plot(rr1)

rr2 <- raster(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20-minus1/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/",
                     "2000-06.tif"))
plot(rr2)

fns_rof <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-rofental-cp20-minus1/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/")

rr_rof <- stack(fns_rof[1:30])
rr_rof <- stack(fns_rof[170:188])
rr_rof
rr_rof %>% plot



# check r scf_th ----------------------------------------------------------

rr0 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/zz_py_temp/test-ds-cp20/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/2000-01.tif")
rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/zz_py_temp/test-ds-cp20-minus1/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/2000-01.tif")
rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/2000-01.tif")
rrm <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD_monthly/hydro_year_2001_2019/01_Jan.tif")

plot(rr0)
plot(rr1)
plot(rr2)
plot(rrm)


# compare r to py

rr_clim_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rrs_clim_r <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/clm_past_clim.tif")
rrs_clim_py <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/zz_py_temp/test-ds-cp20/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")

dat_r <- data.table(scd_modis = rr_clim_modis[],
                    scd_r = rrs_clim_r[],
                    scd_py = rrs_clim_py[])

na.omit(dat_r) %>% 
  ggplot(aes(scd_modis, scd_r))+
  # geom_bin2d()+
  geom_bin2d(binwidth = 10)+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")

na.omit(dat_r) %>% 
  ggplot(aes(scd_modis, scd_r - scd_modis))+
  # geom_bin2d()+
  geom_bin2d(binwidth = 10)+
  geom_hline(yintercept = 0)+
  scale_fill_viridis_c(trans = "log10")

na.omit(dat_r) %>% 
  .[, .(bias_r_modis = mean(scd_r - scd_modis),
        bias_py_modis = mean(scd_py - scd_modis),
        bias_py_r = mean(scd_py - scd_r),
        mae_r_modis = mean(abs(scd_r - scd_modis)),
        mae_py_modis = mean(abs(scd_py - scd_modis)),
        mae_py_r = mean(abs(scd_py - scd_r)))] -> dt_r_py_summ

dt_r_py_summ

# compare past to modis

paths_todo <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/", 
                     type = "directory")

for(i in seq_along(paths_todo)){
  
  fn_out <- paths_todo[i] %>% 
    path_file %>% 
    path_ext_set("tif") %>% 
    path("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/", .)
  
  calc_clim_ds(paths_todo[i], fn_out)
  
}


rr1_f <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.tif")
rr1_p <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")
rr2_f <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/CLMcom-CCLM4-8-17_ICHEC-EC-EARTH_rcp26_r12i1p1_v1_future.tif")



plot(rr1_f, rr_clim_modis)
abline(0,1,col="red")

plot(rr1_p, rr_clim_modis)
abline(0,1,col="red")

plot(rr2_f, rr_clim_modis)
abline(0,1,col="red")

dat_r <- data.table(scd_modis = rr_clim_modis[],
                       scd_past = rr1_p[],
                       scd_fut85 = rr1_f[],
                       scd_fut26 = rr2_f[])

na.omit(dat_r) %>% 
  ggplot(aes(scd_modis, scd_past))+
  geom_bin2d(binwidth=10)+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")

na.omit(dat_r) %>% 
  ggplot(aes(scd_modis, scd_fut85))+
  geom_bin2d(binwidth=10)+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")

na.omit(dat_r) %>% 
  ggplot(aes(scd_modis, scd_fut26))+
  geom_bin2d(binwidth=10)+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")


na.omit(dat_r) %>% 
  .[, .(bias_past = mean(scd_past - scd_modis),
        bias_fut85 = mean(scd_fut85 - scd_modis),
        bias_fut26 = mean(scd_fut26 - scd_modis),
        mae_past = mean(abs(scd_past - scd_modis)),
        mae_fut85 = mean(abs(scd_fut85 - scd_modis)),
        mae_fut26 = mean(abs(scd_fut26 - scd_modis)))] -> dt_r_summ

dt_r_summ

rr_diff <- rr1_p - rr_clim_modis
plot(rr_diff)


# check rofental ----------------------------------------------------------

path_rof <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r_rofental/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past/"

files1 <- path(path_rof, sprintf("2002-%02i-01.tif", 1:12))
rs1 <- stack(files1)
plot(rs1, zlim = c(-1,1))

