# taken from package qmap
# functions fitQmapQUANT and doQmapQUANT


# changes made:
#
# - replaced wet.day with trace and trace.calc, see MBC::QDM()
#
#



QM_snc <- function(obs, mod.c, mod.p, 
                   trace = 0.05, trace.calc = 0.5 * trace, 
                   qstep = 0.01){
  
  
  # get max possible snow cover if != 1, but only if slightly lower (some RCM have e.g max=0.98)
  max_snc <- max(mod.c)
  if(max_snc <= 0.95) max_snc <- 1
  
  # add random component (to deal with constant 0 and 1 values)
  epsilon <- .Machine$double.eps
  # 0 limit
  obs[obs < trace.calc] <- runif(sum(obs < trace.calc), min = epsilon, max = trace.calc)
  mod.c[mod.c < trace.calc] <- runif(sum(mod.c < trace.calc), min = epsilon, max = trace.calc)
  mod.p[mod.p < trace.calc] <- runif(sum(mod.p < trace.calc), min = epsilon, max = trace.calc)
  # 1 (or max) limit 
  obs[(max_snc - obs) < trace.calc] <- max_snc - runif(sum((max_snc - obs) < trace.calc), min = epsilon, max = trace.calc)
  mod.c[(max_snc - mod.c) < trace.calc] <- max_snc - runif(sum((max_snc - mod.c) < trace.calc), min = epsilon, max = trace.calc)
  mod.p[(max_snc - mod.p) < trace.calc] <- max_snc - runif(sum((max_snc - mod.p) < trace.calc), min = epsilon, max = trace.calc)
  
  
  # original qm code
  hn <- min(length(obs), length(mod.c))
  quant.obs <- quantile(obs, seq(0, 1, length.out = hn), type = 8)
  quant.mod.c <- quantile(mod.c, seq(0, 1, length.out = hn), type = 8)
  
  modq <- quantile(quant.mod.c, probs = seq(0, 1, by = qstep), type = 8)
  fitq <- quantile(quant.obs, probs = seq(0, 1, by = qstep), type = 8)
  
  nq <- length(modq)
  
  mod.hat.c <- approx(x = modq, y = fitq, xout = mod.c,
                      method = "linear", rule = 2, 
                      ties = mean)$y
  
  largex <- mod.c > modq[nq]
  if (any(largex)) {
    max.delta <- modq[nq] - fitq[nq]
    mod.hat.c[largex] <- mod.c[largex] - max.delta
  }					  
  
  mod.hat.p <- approx(x = modq, y = fitq, xout = mod.p,
                      method = "linear", rule = 2, 
                      ties = mean)$y
  
  largex <- mod.p > modq[nq]
  if (any(largex)) {
    max.delta <- modq[nq] - fitq[nq]
    mod.hat.p[largex] <- mod.p[largex] - max.delta
  }			
  
  
  # revert random component for trace
  # 0 limit
  mod.hat.c[mod.hat.c < trace] <- 0
  mod.hat.p[mod.hat.p < trace] <- 0
  # 1 (or max) limit
  mod.hat.c[(max_snc - mod.hat.c) < trace] <- max_snc
  mod.hat.p[(max_snc - mod.hat.p) < trace] <- max_snc
  
  
  return(list(mod.hat.c = mod.hat.c,
              mod.hat.p = mod.hat.p))
  
}





