## convert netcdf-raster to data.table ####
# use `as.data.frame,Raster-method`

# params
#   : xy - as in above method
#   : icell - adds interger of cell id
#   : remove.na - remove all NA values
#   : subset.date - list(from=, to=) to subset netcdf-raster by date
#                   in ymd() readable format
#   : subset.icell - vector of cell indices to subset


require(data.table)
require(raster)
require(magrittr)
require(lubridate)
require(progress)

raster2datatable <- function(rr, 
                             xy = F, icell = F, remove.na = F,
                             subset.date = NULL, subset.icell = NULL){
  
  if(!is.null(subset.date)){
    
    # get date-layer information
    dt_date <- data.table(xdate = names(rr))
    dt_date[, z := 1:.N]
    dt_date[, date := ymd(substr(xdate, 2, 99))]
    
    # subset z
    z_date_sel <- dt_date[date >= ymd(subset.date$from) & date <= ymd(subset.date$to), 
                          z]
  
    
    if(is.null(subset.icell)){
      # subset whole raster by layer
      # with progress bar
      pb <- progress_bar$new(total = length(z_date_sel),
                             clear = F,
                             format = "Extracting from netcdf file [:bar] :percent eta: :eta")
      if(xy) print("xy set to FALSE, not implemented for TRUE")
      l <- lapply(z_date_sel, function(z) {
        res <- as.data.frame(rr[[z]], xy = F)
        setDT(res)
        pb$tick()
        res
      })
      
      dat <- do.call(cbind, l)
      
    } else {
      # subset icell all layers
      dat <- data.table(rr[[z_date_sel]][subset.icell])
      
    }
    
  } else {
    
    if(is.null(subset.icell)){
      # take whole raster
      rr %>% 
        as.data.frame(xy = xy) %>% 
        data.table -> dat
      
    } else {
      # subset icell all layers
      dat <- data.table(rr[subset.icell])
      
    }
    
  }
  

  if(icell) {
    if(is.null(subset.icell)){
      dat[, icell := 1:ncell(rr)]
    } else {
      dat[, icell := subset.icell]
    }
    
  }
  if(remove.na) dat <- na.omit(dat)
  dat[]
}