# plot maps of SCD past future

library(raster)
library(stars)
library(ggplot2)
library(scico)
library(data.table)
library(magrittr)
library(fs)
library(graticule)

library(cowplot)

# ds_stars <- 10 # for template
ds_stars <- 5 # for production 2-5







# map scd with change -----------------------------------------------------


rs_modis <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rs_rcp26 <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp26.tif")
rs_rcp85 <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp85.tif")





## graticules --------------------------------------------------------------

prj <- st_crs(rs_modis)$proj4string

lons <- seq(10, 12, by = 2)
lats <- seq(46, 48, by = 1)
xl <- range(lons) + c(-0.4, 0.4)
yl <- range(lats) + c(-0.4, 0.4)
grat <- graticule(lons, lats, proj = prj, xlim = xl, ylim = yl)
labs <- graticule_labels(lons, lats, xline = min(xl), yline = max(yl), proj = prj)



## plot and combine ---------------------------------------------------------

lim_diff <- range(rs_rcp26[[1]] - rs_modis[[1]], 
                  rs_rcp85[[1]] - rs_modis[[1]],
                  na.rm = T)

# gg1 <-
# ggplot()+
#   geom_sf(data = st_as_sf(grat))+
#   geom_stars(data = rs_modis, downsample = ds_stars)+
#   scale_fill_scico("SCD [d]", limits = c(0, 366),
#                    palette = "devon", na.value = NA, direction = 1)+
#   coord_sf(expand = F)+
#   theme_minimal()+
#   xlab(NULL)+ylab(NULL)+
#   scale_x_continuous(labels = function(x) paste0(x, "° E"))+
#   scale_y_continuous(labels = function(x) paste0(x, "° N"))

gg_rcp26 <-
ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_rcp26, downsample = ds_stars)+
  scale_fill_scico("SCD [d]", limits = c(0, 366),
                   palette = "devon", na.value = NA, direction = 1)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))+
  facet_wrap(~"RCP2.6")

gg_rcp85 <-
ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_rcp85, downsample = ds_stars)+
  scale_fill_scico("SCD [d]", limits = c(0, 366),
                   palette = "devon", na.value = NA, direction = 1)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))+
  facet_wrap(~"RCP8.5")

gg_rcp26_diff <-
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = (rs_rcp26 - rs_modis), downsample = ds_stars)+
  scale_fill_scico("dSCD [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   rescaler = scales::rescale_mid,
                   limits = lim_diff)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))+
  facet_wrap(~"RCP2.6")

gg_rcp85_diff <-
  ggplot()+
    geom_sf(data = st_as_sf(grat), colour = NA)+
    geom_stars(data = rs_rcp85 - rs_modis, downsample = ds_stars)+
    scale_fill_scico("dSCD [d]", 
                     palette = "vik", na.value = NA, direction = -1,
                     rescaler = scales::rescale_mid,
                     limits = lim_diff)+
    coord_sf(expand = F)+
    theme_minimal()+
    xlab(NULL)+ylab(NULL)+
    scale_x_continuous(labels = function(x) paste0(x, "° E"))+
    scale_y_continuous(labels = function(x) paste0(x, "° N"))+
  facet_wrap(~"RCP8.5")


gg_out <- plot_grid(gg_rcp26+theme(legend.position = "none"), 
                    gg_rcp85,
                    gg_rcp26_diff+theme(legend.position = "none"), 
                    gg_rcp85_diff,
                    rel_widths = c(1, 1.1),
                    labels = c("(a)", "", "(b)", ""))

ggsave(gg_out,
       filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/ds-map.png",
       width = 11,
       height = 6)  





# suppl: elev anomaly -----------------------------------------------------

rs_modis_anom <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD_anomaly_elev/2000-10-01_2019-09-30_bin100m.tif")
rs_rcp26_anom <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp26_elev_anom.tif")
rs_rcp85_anom <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp85_elev_anom.tif")

lim_anom <- range(rs_modis_anom[[1]],
                  rs_rcp26_anom[[1]],
                  rs_rcp85_anom[[1]],
                  na.rm = T)

# lim_anom <- NULL

gg_anom_modis <-
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_modis_anom, downsample = ds_stars)+
  scale_fill_scico("dSCD [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   rescaler = scales::rescale_mid,
                   limits = lim_anom)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))+
  ggtitle("2000-2020 MODIS")

gg_anom_rcp26 <-
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_rcp26_anom, downsample = ds_stars)+
  scale_fill_scico("", 
                   palette = "vik", na.value = NA, direction = -1,
                   rescaler = scales::rescale_mid,
                   limits = lim_anom)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))+
  ggtitle("2071-2100 RCP2.6")



gg_anom_rcp85 <-
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_rcp85_anom, downsample = ds_stars)+
  scale_fill_scico("SCD elev. anom. [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   rescaler = scales::rescale_mid,
                   limits = lim_anom)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))+
  ggtitle("2071-2100 RCP8.5")


gg_out <-
  plot_grid(gg_anom_modis + theme(legend.position = "none"),
            gg_anom_rcp26 + theme(legend.position = "none"),
            gg_anom_rcp85 + theme(legend.position = "bottom",
                                  legend.key.width = unit(1, "cm")),
            rel_heights = c(1, 1.3),
            labels = c("(a)", "(b)", "(c)"))

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/ds-map-elev-anom.png",
       gg_out,
       width = 9,
       height = 6)
