
library(raster)
library(data.table)
library(ggplot2)
library(magrittr)
library(foreach)
library(forcats)




# get sample pixel data ---------------------------------------------------


icell_sample <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/sample-pixels-01-stratified-elev.rds")

dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")
dat_modis <- dat_modis[icell %in% icell_sample]



c(
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/02_qm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/03_qdm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/04_mbc_qdm/"
) %>% 
  lapply(function(x){
    
    data.table(bc_var = path_file(x),
               fn = dir_ls(x))
    
  }) %>% rbindlist -> dat_loop

dat_loop[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
           tstrsplit(path_ext_remove(path_file(fn)), "_")]


dat_loop2 <- dat_loop[institute_rcm == "SMHI-RCA4" & gcm == "ICHEC-EC-EARTH"]
# dat_loop2 <- dat_loop[institute_rcm == "CNRM-ALADIN63" & gcm == "MPI-M-MPI-ESM-LR"]


dat_rcm_sample <- foreach(
  i = seq_len(nrow(dat_loop2)),
  .final = rbindlist
) %do% {
  
  dat <- readRDS(dat_loop2[i, fn])
  dat2 <- dat[icell %in% icell_sample]
  setnames(dat2, 5, "snc_bc")
  
  dat2 %>% cbind(dat_loop2[i, .(bc_var, fp)])
}






# choose an icell ---------------------------------------------------------

rr0 <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
plot(rr0)

rr1 <- rr0
rr1[icell_sample] <- 10
# rr1[2678] <- 20
rr1[2842] <- 20
# rr1[sort(icell_sample)[2]] <- 20
plot(rr1)


sort(icell_sample)[2]

dat_modis %>% ggplot(aes(snc))+stat_ecdf()+facet_wrap(~icell)

# plot --------------------------------------------------------------------

dat_plot_modis <- dat_modis[icell == 2215]
dat_plot <- dat_rcm_sample[icell == 2215 & bc_var != "04_mbc_qdm"]

dat_plot2 <- rbind(dat_plot[, .(icell, date, bc_var = "01_raw", snc, fp)],
                   dat_plot[, .(icell, date, bc_var, snc = snc_bc, fp)])

dat_plot2[, bc_var_f := fct_recode(factor(bc_var),
                                   "RAW" = "01_raw",
                                   "QM" = "02_qm",
                                   "QDM" = "03_qdm")]


dat_plot2 %>% 
  ggplot(aes(snc, colour = bc_var_f, linetype = fp))+
  stat_ecdf(pad = F)+
  stat_ecdf(pad = F,
            data = dat_plot_modis, 
            aes(colour = NULL, linetype = NULL),
            linetype = "dotted")+
  # facet_wrap(~ bc_var)+
  scale_linetype("Period")+
  scale_color_discrete("RCM")+
  theme_bw()+
  ylim(0,1)+xlim(0,1)+
  xlab("Snow cover fraction [0-1]")+
  ylab("ECDF (empirical cumulative probability)")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-pixel.png",
       width = 6,
       height = 4)
