# compare past downscale to MODIS SCD

library(data.table)
library(magrittr)
# library(raster)
library(stars)
library(sf)
library(ggplot2)
library(fs)
library(foreach)
library(forcats)


rr_modis <- raster::raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")

rs_modis <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rs_th <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v05_impute_level.tif")
rs_rcm_past <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")


rs_elev <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rs_elev[rs_elev < -30000] <- NA

rs_corine <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif")
rs_elev[rs_corine[,,,3] == 0] <- NA


rs_rgi <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")

# ds_stars <- 10


# example -----------------------------------------------------------------


dat1 <- data.table(scd_modis = as.vector(rs_modis[[1]]),
                   scd_past = as.vector(rs_rcm_past[[1]]),
                   th_level = as.vector(rs_th[[1]]),
                   frac_glacier = as.vector(rs_rgi[[1]]),
                   elev = as.vector(rs_elev[[1]]))
dat2 <- dat1[!is.na(th_level)]
dat2[is.na(frac_glacier), frac_glacier := 0]

dat2$elev %>% summary
dat2[, elev_f := cut(elev, breaks = c(-50, seq(500, 4500, by = 500)), dig.lab = 5)]
dat2[, th_level_f := fct_recode(factor(th_level),
                                "Original" = "1",
                                "Impute: similar prob. curves" = "2",
                                "Impute: elevation" = "3")]
dat2[frac_glacier > 0, qplot(frac_glacier)]
dat2[, frac_glacier_f := ifelse(frac_glacier < 0.1, "<10% glacier in pixel", ">=10% glacier in pixel")]

# 
# ggplot(dat2, 
#        aes(scd_modis, scd_past))+
#   geom_bin2d(binwidth = 20)+
#   scale_fill_viridis_c(trans = "log10")+
#   geom_abline()+
#   facet_wrap(~th_level)+
#   theme_bw()
# 

gg <- ggplot(dat2, 
       aes(elev, scd_past - scd_modis))+
  geom_bin2d(binwidth = c(100, 5))+
  scale_fill_viridis_c(trans = "log10")+
  geom_hline(yintercept = 0)+
  facet_grid(frac_glacier_f~th_level_f)+
  theme_bw()+
  xlab("Elevation [m]")+
  ylab("SCD difference (RCM - MODIS) [d]")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/downscale-bias-glacier.png",
       gg,
       width = 12,
       height = 6)



# ggplot(dat2, 
#        aes(scd_past - scd_modis, elev_f, fill = factor(th_level)))+
#   geom_vline(xintercept = 0)+
#   geom_boxplot(coef = Inf)+
#   theme_bw()


dat3 <- dat2[, 
             .(bias = mean(scd_past - scd_modis, na.rm = T),
               mae = mean(abs(scd_past - scd_modis), na.rm = T),
               nn = .N),
             .(elev_f, th_level, frac_glacier_f)]

  
dat3 %>% 
  ggplot(aes(bias, elev_f, colour = factor(th_level)))+
  geom_vline(xintercept = 0)+
  geom_point()+
  facet_wrap(~frac_glacier_f)+
  theme_bw()


dat3 %>% 
  ggplot(aes(mae, elev_f, colour = factor(th_level)))+
  geom_vline(xintercept = 0)+
  geom_point()+
  facet_wrap(~frac_glacier_f)+
  theme_bw()



dat3b <- dat2[, 
             .(bias = mean(scd_past - scd_modis, na.rm = T),
               mae = mean(abs(scd_past - scd_modis), na.rm = T),
               nn = .N),
             keyby = .(frac_glacier_f, elev_f)]


# numbers -----------------------------------------------------------------

dat_nn_glacier <- dat2[, 
                       .(bias = mean(scd_past - scd_modis, na.rm = T),
                         mae = mean(abs(scd_past - scd_modis), na.rm = T),
                         nn = .N),
                       keyby = .(th_level, frac_glacier_f)]
dat_nn_glacier

dat_nn_glacier[, weighted.mean(bias, nn)]
dat_nn_glacier[, weighted.mean(mae, nn)]


dat_nn_glacier[frac_glacier_f == "<10% glacier in pixel", weighted.mean(bias, nn)]
dat_nn_glacier[frac_glacier_f == "<10% glacier in pixel", weighted.mean(mae, nn)]