# ds (only qdm)
# elev plot


library(data.table)
library(magrittr)
library(raster)
# library(stars)
library(ggplot2)
library(fs)
library(foreach)

library(flextable)
library(officer)




# common icell ------------------------------------------------------------

dat_icell <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/common-icell.rds")
names(dat_icell)
rcms_common <- c("MODIS", "CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1",
                 "CNRM-ALADIN63", "IPSL-WRF381P", "SMHI-RCA4")
lgl_icell_common <- dat_icell[, 
                              apply(.SD, 1, all), 
                              .SDcols = rcms_common]


# glacier -----------------------------------------------------------------


rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")
lgl_icell_noglacier <- rr_rgi[] <= 0.1

# modis data --------------------------------------------------------------

rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")


dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd = rr_modis[],
                        elev = rr_elev[])
# dat_modis <- dat_modis[!is.na(scd)]
# dat_modis$elev %>% qplot
# dat_modis[lgl_icell_common & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier, elev] %>% summary

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 3600, by = 200), 4200), dig.lab = 5)]

dat_modis2 <- dat_modis[lgl_icell_common & lgl_icell_noglacier, 
                        .(scd = mean(scd),
                          nn = .N),
                        .(elev_f)]

dat_modis2 %>% ggplot(aes(scd, elev_f))+geom_point()


# qdm data ----------------------------------------------------------------

x <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/"
dat_loop <- data.table(bc_var = path_file(x),
                       file_rcm = dir_ls(x))

dat_loop[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
           tstrsplit(path_ext_remove(path_file(file_rcm)), "_")]

# dat_loop2 <- dat_loop[institute_rcm %in% rcms_common & fp == "future"]
dat_loop2 <- dat_loop[institute_rcm %in% rcms_common]


dat_rcm2 <- foreach(
  i = seq_len(nrow(dat_loop2)),
  .final = rbindlist
) %do% {
  
  # i_rcm <- dat_loop_rcm[i, institute_rcm]

  i_rr <- raster(dat_loop2[i, file_rcm])
  
  dat_i <- data.table(dat_modis[, .(icell, elev, elev_f)],
                      scd = i_rr[])
  
  dat_i2 <- dat_i[lgl_icell_common & lgl_icell_noglacier, 
                      .(scd = mean(scd),
                        nn = .N),
                      .(elev_f)]
  
  dat_i3 <- dat_i2 %>% 
    cbind(dat_loop2[i, ])
  dat_i3[, file_rcm := NULL]
  dat_i3
  
  
}



# plot --------------------------------------------------------------------

dat_plot <- copy(dat_rcm2)

rcm_short <- readRDS("data/auxiliary-01-rcm-names.rds")
dat_plot[, rcm_short := rcm_short[institute_rcm]]

dat_plot[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]
dat_plot[, lbl_time := ifelse(fp == "past", 
                              "2000-2020",
                              "2071-2100")]

setorder(dat_plot, rcm_short, gcm, lbl_time, experiment_f, elev_f)
dat_plot$elev_f %>% levels

# dat_plot[, elev_f_out := ifelse(elev_f %in%c("(3200,3400]", "(3400,3600]", "(3600,4200]"),
#                                 "above",
#                                 "within")]

gg <- dat_plot %>% 
  ggplot(aes(scd, elev_f))+
  geom_point(data = dat_modis2, aes(shape = "MODIS"))+
  geom_path(aes(colour = rcm_short, linetype = gcm, 
                # alpha = elev_f_out,
                group = paste0(rcm_short,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  # scale_alpha_manual("RCM elev.", values = c(0.4, 1))+
  scale_shape("2000-2020 Obs.")+
  facet_grid(lbl_time ~ experiment_f)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Snow cover duration [days]")+
  ylab("Elevation [m]")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/ds-elev.png",
       gg, 
       width = 11,
       height = 8)



# plot diff ---------------------------------------------------------------

# diff to modis

dat_plot_diff <- dat_plot %>%
  merge(dat_modis2[, .(elev_f, scd_ref = scd)])

# or diff to past?
# dat_plot_diff <- dat_plot[fp == "future"] %>%
#   merge(dat_plot[fp == "past", .(ff, month_fct, rcm, gcm, snc_ref = snc)])

dat_plot_diff[, scd_diff := scd - scd_ref]
dat_plot_diff[, scd_ratio := scd / scd_ref]

gg <-
dat_plot_diff %>%
  ggplot(aes(scd_diff, elev_f))+
  geom_vline(xintercept = 0)+
  geom_path(aes(colour = rcm_short, linetype = gcm, 
                # alpha = elev_f_out,
                group = paste0(rcm_short,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  # scale_alpha_manual("RCM elev.", values = c(0.4, 1))+
  facet_grid(lbl_time ~ experiment_f)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Difference in snow cover duration w.r.t. observations [days]")+
  ylab("Elevation [m]")



ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/ds-elev-diff.png",
       gg, 
       width = 11,
       height = 8)



# table -------------------------------------------------------------------

dat_modis2

dat_table <- dat_plot[,
                      .(mod_mean = mean(scd),
                        mod_min = min(scd),
                        mod_max = max(scd),
                        mod_n = .N),
                      .(elev_f, lbl_time, experiment_f)]

dat_table[, mmm := paste0(round(mod_mean),
                          " (", round(mod_min), ", ", round(mod_max), ")")]

dat_table2 <- dcast(dat_table[lbl_time == "2071-2100"],
                    elev_f ~ experiment_f, 
                    value.var = "mmm")
dat_table2 %>% 
  merge(dat_modis2[, .(elev_f, MODIS = round(scd))]) -> dat_table3

setorder(dat_table3, -elev_f)

dat_table3 %>% 
  flextable(col_keys = c("elev_f", "MODIS", "RCP2.6", "RCP8.5")) %>% 
  set_header_labels(elev_f = "Elevation [m]",
                    MODIS = "MODIS 2000-2020",
                    RCP2.6 = "RCP2.6 2071-2100",
                    RCP8.5 = "RCP8.5 2071-2100") %>% 
  align(align = "center") %>% 
  # fontsize(size = 6) %>% 
  autofit -> ft

read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/ds-elev.docx")




# table diff --------------------------------------------------------------

# also past for writing?



# numbers -----------------------------------------------------------------


dat_table %>% 
  merge(dat_modis2) %>% 
  .[lbl_time == "2071-2100", 
    .(elev_f, experiment_f, d_perc = round(100*(scd - mod_mean) / scd))]


