# compare flo data to ds

library(stars)
# library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(fs)

rr_template_ds <- raster("data/rofental_modis_laea_flo.tif")
rs_template_ds <- read_stars("data/rofental_modis_laea_flo.tif")


rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")
rr_rgi_rofental <- crop(rr_rgi, rr_template_ds)

# overview ds ------------------------------------------------------------

dat_ds <- data.table(fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/"))
dat_ds[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]


# rcms without snow accumulation in rofental
# dat_ds <- dat_ds[experiment == "rcp85" & institute_rcm %in% c(
#   "CLMcom-CCLM4-8-17", 
#   "CLMcom-ETH-COSMO-crCLIM-v1-1",
#   "IPSL-WRF381P"
# )]


# all other in paper
dat_ds <- dat_ds[experiment == "rcp85" & institute_rcm %in% c(
  "CLMcom-CCLM4-8-17", 
  "CLMcom-ETH-COSMO-crCLIM-v1-1",
  # "CNRM-ALADIN63", # 7 lr pixels missing
  "IPSL-WRF381P",
  "SMHI-RCA4" # 2 lr pixels missing
)]


# overview amundsen -------------------------------------------------------


x <- "data-aux/amundsen-flo-hr/maps/"
dat_am <- data.table(fn = dir_ls(x))
dat_am[, c("zz", "gcm", "institute_rcm", "experiment", "period") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]
dat_am[, fp := ifelse(period == "2000-2020", "past", "future")]

dat_am <- dat_am[experiment == "rcp85"]



# ensemble means ----------------------------------------------------------



rr_am_past <- stack(dat_am[fp == "past", fn])
rr_am_past_mean <- calc(rr_am_past, mean)
rr_am_past_mean

rr_am_future <- stack(dat_am[fp == "future", fn])
rr_am_future_mean <- calc(rr_am_future, mean)
rr_am_future_mean


rr_ds_past <- stack(dat_ds[fp == "past", fn])
rr_ds_past <- crop(rr_ds_past, rr_template_ds)
rr_ds_past_mean <- calc(rr_ds_past, mean)
rr_ds_past_mean_noglacier <- rr_ds_past_mean
rr_ds_past_mean_noglacier[rr_rgi_rofental > 0.1] <- NA

rr_ds_future <- stack(dat_ds[fp == "future", fn])
rr_ds_future <- crop(rr_ds_future, rr_template_ds)
rr_ds_future_mean <- calc(rr_ds_future, mean)
rr_ds_future_mean_noglacier <- rr_ds_future_mean
rr_ds_future_mean_noglacier[rr_rgi_rofental > 0.1] <- NA





# method diff raster ------------------------------------------------------


rr_past_diff <- rr_am_past_mean*365.25 - projectRaster(rr_ds_past_mean_noglacier, rr_am_past_mean)
rr_future_diff <- rr_am_future_mean*365.25 - projectRaster(rr_ds_future_mean_noglacier, rr_am_future_mean)


# graticules --------------------------------------------------------------

prj <- st_crs(rs_template_ds)$proj4string

lons <- seq(10, 12, by = 2)
lats <- seq(46, 48, by = 1)
xl <- range(lons) + c(-0.4, 0.4)
yl <- range(lats) + c(-0.4, 0.4)
grat <- graticule(lons, lats, proj = prj, xlim = xl, ylim = yl)
labs <- graticule_labels(lons, lats, xline = min(xl), yline = max(yl), proj = prj)

grat_am <- graticule(lons, lats, proj = st_crs(rr_am_future)$proj4string, xlim = xl, ylim = yl)

# plot --------------------------------------------------------------------




lim_diff <- range(rr_ds_future_mean_noglacier[] - rr_ds_past_mean_noglacier[], 
                  (rr_am_future_mean[] - rr_am_past_mean[])*365.25,
                  na.rm  =T)

lim_diff_method <- range(rr_past_diff[], rr_future_diff[], 
                         na.rm = T)


gg_am_past <- ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat_am), rr_am_future_mean), colour = NA)+
  geom_stars(data = st_as_stars(rr_am_past_mean*365.25))+
  scale_fill_scico("SCD [d]", palette = "davos", limits = c(0,366))+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  facet_wrap(~"AMUNDSEN 2000-2020")

gg_am_future <- ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat_am), rr_am_future_mean), colour = NA)+
  geom_stars(data = st_as_stars(rr_am_future_mean*365.25))+
  scale_fill_scico("SCD [d]", palette = "davos", limits = c(0,366))+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  facet_wrap(~"AMUNDSEN 2071-2100")


gg_ds_past <- ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat), rs_template_ds), colour = NA)+
  geom_stars(data = st_as_stars(rr_ds_past_mean_noglacier))+
  scale_fill_scico("SCD [d]", palette = "davos", limits = c(0,366),
                   na.value = NA)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  facet_wrap(~"Downscaling 2000-2020")

gg_ds_future <- ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat), rs_template_ds), colour = NA)+
  geom_stars(data = st_as_stars(rr_ds_future_mean_noglacier))+
  scale_fill_scico("SCD [d]", palette = "davos", limits = c(0,366),
                   na.value = NA)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  facet_wrap(~"Downscaling 2071-2100")

# period diff

gg_am_diff <- ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat_am), rr_am_future_mean), colour = NA)+
  geom_stars(data = st_as_stars((rr_am_future_mean - rr_am_past_mean)*365.25))+
  scale_fill_scico("dSCD [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   rescaler = scales::rescale_mid,
                   limits = lim_diff)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  facet_wrap(~"Future - past (AMUNDSEN)")

gg_ds_diff <- ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat), rs_template_ds), colour = NA)+
  geom_stars(data = st_as_stars((rr_ds_future_mean_noglacier - rr_ds_past_mean_noglacier)))+
  scale_fill_scico("dSCD [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   rescaler = scales::rescale_mid,
                   limits = lim_diff)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  facet_wrap(~"Future - past (Downscaling)")


# method diff


gg_past_diff <- ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat_am), rr_am_future_mean), colour = NA)+
  geom_stars(data = st_as_stars(rr_past_diff))+
  scale_fill_scico("dSCD [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   rescaler = scales::rescale_mid,
                   limits = lim_diff_method)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  facet_wrap(~"AMUNDSEN - Downscaling (past)")

gg_future_diff <- ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat_am), rr_am_future_mean), colour = NA)+
  geom_stars(data = st_as_stars(rr_future_diff))+
  scale_fill_scico("dSCD [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   rescaler = scales::rescale_mid,
                   limits = lim_diff_method)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  facet_wrap(~"AMUNDSEN - Downscaling (future)")







gg_out <- 
  plot_grid(gg_am_past, gg_ds_past, gg_past_diff,
            gg_am_future, gg_ds_future, gg_future_diff,
            gg_am_diff, gg_ds_diff,
            ncol = 3,
            # rel_widths = c(1, 1.2, 1.3),
            labels = paste0("(", letters[1:8], ")"))

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/rofental-map.png",
       gg_out,
       width = 12,
       height = 8)
