# get delta-change data for UIBK


library(data.table)
library(magrittr)
library(raster)
library(sf)
library(ggplot2)


source("R/functions/raster2datatable.R")

dat_xy <- readRDS("data/auxiliary-03-modis.rds")

# get icell for province BZ -----------------------------------------------

sf_ita <- read_sf("/mnt/CEPH_BASEDATA/GIS/NATIONAL/ITALY/BOUNDARIES/IT_bnd_prov_pol_ISTAT_2011.shp")
sf_bz <- sf_ita %>% dplyr::filter(NOME_PRO == "BOLZANO/BOZEN")

rr_coarse <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-03-03.tif")
sf_bz2 <- st_transform(sf_bz, st_crs(rr_coarse))

# rr_bz <- rasterize(as(sf_bz2, "Spatial"), rr_coarse)
rr_bz <- rasterize(as(sf_bz2, "Spatial"), rr_coarse, getCover = T)


dat_bz <- raster2datatable(rr_bz, icell = T)

dat_xy %>% 
  merge(dat_bz[, .(icell, bz_cover = layer)]) -> dat_xy_bz


# get rcm data ------------------------------------------------------------

# zz <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/01_delta_change/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1.rds")

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-01.rda")


dat1 <- merge(dat_delta2, dat_xy_bz[bz_cover > 0], by = "icell")
mitmatmisc::add_month_fct(dat1, 10)

dat1[rcm == "CLMcom-CCLM4-8-17"] %>% 
  ggplot(aes(alt, snc_future_delta))+
  geom_point()+
  facet_wrap(~month)+
  theme_bw()


dat1[, .N, .(rcm, gcm, experiment)]


dat2 <- dat1[, .(month, month_chr = month_fct, rcm, gcm, experiment, icell, x, y, alt, bz_cover,
                 snc_modis_2000_2020 = snc,
                 snc_rcm_2071_2100_bc = snc_future_delta,
                 snc_delta = snc_future_delta - snc,
                 raw_snc_rcm_2000_2020 = snc_past,
                 raw_snc_rcm_2071_2100 = snc_future)]


dat2[, .(rcm, gcm, experiment)] %>% 
  unique()

# get ensemble (same number)
dat2[experiment == "rcp26", .(rcm, gcm)] %>% 
  unique %>% 
  merge(dat2) -> dat2_sub
  


# dat2_sub %>% 
#   ggplot(aes(alt, snc_rcm_2071_2100_bc, colour = paste0(rcm, "_", gcm)))+
#   geom_point(size = 0.2, alpha = 0.5)+
#   geom_smooth(se = F)+
#   # facet_grid(rcm+gcm~month)+
#   facet_wrap(~month)+
#   theme_bw()


dat2_sub %>% 
  ggplot(aes(alt, snc_delta, colour = experiment))+
  geom_point(size = 1)+
  geom_smooth(se = F)+
  facet_grid(rcm+gcm~month_chr)+
  # facet_wrap(~month)+
  theme_bw()

ggsave(filename = "export-uibk/fig_delta_6gcmrcm.pdf",
       width = 22, height = 11)


dat2


fwrite(dat2,
       "export-uibk/snc_delta_bz.csv")



# save tif for spatial ----------------------------------------------------

writeRaster(rr_coarse, "export-uibk/example-tif_modis-snc-20000303.tif")


# aux plots ---------------------------------------------------------------

dat_xy_bz[bz_cover > 0] %>% 
  ggplot(aes(x, y, fill = alt))+
  geom_raster()+
  scale_fill_viridis_c()+
  theme_bw()


mapview::mapview(rr_bz)


